@@ -1,6 +1,11 @@
 /***************************************************************************
  *   Copyright (C) 2004-2005 by Daniel Clarke <daniel.jc@gmail.com>        *
  *                      2005 by David Saxton <david@bluehaze.org>          *
+ *									   *
+ *   24-04-2007                                                            *
+ *   Modified to add pic 16f877,16f627 and 16f628 			   *
+ *   by george john george@space-kerala.org 				   *
+ *   supported by SPACE www.space-kerala.org	 			   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -62,18 +67,66 @@ class Register
 			EECON2,
 			PCLATH,
 			INTCON,
-			
+//modification start
+			PORTC,
+			PORTD,
+			PORTE,
+			TRISC,
+			TRISD,
+			TRISE,
+			ADCON0,
+			ADCON1,
+//modification end			
 			// The following three are "special"
 			WORKING, // Not a register that is addressable by an address
 			GPR, // Refers to the collection of General Purpose Registers
-			none, // used in default constructor
+//modification start
+			PIR1,
+			PIR2,
+			TMR1L,
+			TMR1H,
+			T1CON,
+			TMR2,
+			T2CON,
+			RCSTA,
+			TXREG,
+			RCREG,
+			ADRESH,
+			PIE1,
+			TXSTA,
+			ADRESL,
+			EEDATH,
+			EEADRH,
+			SSPBUF,
+			SSPCON,
+			CCPR1L,
+			CCPR1H,
+			CCP1CON,
+			CCPR2L,
+			CCPR2H,
+			CCP2CON,
+			PIE2,
+			PCON,
+			SSPCON2,
+			PR2,
+			SSPADD,
+			SSPSTAT,
+			SPBRG,
+			VRCON,
+			CMCON,
+
+//modification end
+			none // used in default constructor
+//TODO
+//SSPBUF:SSPCON:CCPR1L:CCPR1H:CCP1CON:CCPR2L:CCPR2H:CCP2CON:--FOR BANK0 
+//PIE2:PCON:SSPCON2:PR2:SSPADD:SSPSTAT:SPBRG:--------FOR BANK1
 		};
 		
 		// These banks are used for ORing together in the banks() function
 		enum Banks
 		{
 			Bank0 = 1 << 0,
-			Bank1 = 1 << 1,
+			Bank1 = 1 << 1
 		};
 		
 		/**
@@ -147,7 +200,7 @@ class RegisterBit
 			NOT_TO		= 4, // Time-out
 			RP0			= 5, // Bank Select
 			RP1			= 6,
-			IRP			= 7,
+			IRP			= 7
 		};
 		
 		enum INTCON_bits
@@ -159,28 +212,29 @@ class RegisterBit
 			INTE		= 4,
 			T0IE		= 5,
 			EEIE		= 6,
-			GIE			= 7,
+			GIE		= 7
 		};
 		
 		enum OPTION_bits
 		{
-			PS0			= 0,
-			PS1			= 1,
-			PS2			= 2,
-			PSA			= 3,
+			PS0		= 0,
+			PS1		= 1,
+			PS2		= 2,
+			PSA		= 3,
 			T0SE		= 4,
 			T0CS		= 5,
 			INTEDG		= 6,
-			NOT_RBPU	= 7,
+			NOT_RBPU	= 7
 		};
 		
 		enum EECON1_bits
 		{
-			RD			= 0,
-			WR			= 1,
+			RD		= 0,
+			WR		= 1,
 			WREN		= 2,
 			WRERR		= 3,
 			EEIF		= 4,
+			EEPGD           = 7
 		};
 		/**
 		 * Constructs a bit of the given register type at the given position.
@@ -480,7 +534,7 @@ class Code
 			Middle				= 2, ///< Used for main code
 			Subroutine			= 3, ///< Used for subroutines
 			
-			PositionCount		= 4, ///< This must remain the last item and be the number of valid positions
+			PositionCount		= 4 ///< This must remain the last item and be the number of valid positions
 		};
 		
 		CodeIterator begin();
@@ -635,7 +689,7 @@ class Instruction
 		{
 			Assembly,
 			Raw, // User-inserted assembly
-			Comment,
+			Comment
 		};
 		/**
 		 * Used in optimization. Note that this follows roughly, but not
@@ -668,7 +722,7 @@ class Instruction
 			/**
 			 * The Instruction is not of Assembly InstructionType.
 			 */
-			None,
+			None
 		};
 
 		Instruction();
