@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2005 by David Saxton                               *
+ *   Copyright (C) 2004-2006 by David Saxton                               *
  *   david@bluehaze.org                                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,16 +24,22 @@
 #include "picitem.h"
 #include "resizeoverlay.h"
 
+#include "utils.h"
 #include <cmath>
+#include <cstdlib>
 
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
 
+#include <qcursor.h>
 #include <qpainter.h>
 #include <qtimer.h>
 
+// FIXME: This source file is HUUUGE!!!, contains numerous clases, should be broken down. 
 
+
+//BEGIN class CMManager
 CMManager::CMManager( ItemDocument *itemDocument )
 	: QObject()
 {
@@ -56,10 +62,8 @@ CMManager::CMManager( ItemDocument *itemDocument )
 CMManager::~CMManager()
 {
 	delete m_allowItemScrollTmr;
-	m_allowItemScrollTmr = 0l;
 	delete m_canvasManipulator;
-	m_canvasManipulator = 0l;
-	
+
 	const ManipulatorInfoList::iterator end = m_manipulatorInfoList.end();
 	for ( ManipulatorInfoList::iterator it = m_manipulatorInfoList.begin(); it != end; ++it )
 	{
@@ -107,39 +111,41 @@ void CMManager::mousePressEvent( EventInfo eventInfo )
 	uint itemType=0;
 	uint cnItemType=0;
 	
+	QCanvasItem * qcanvasItem = eventInfo.qcanvasItemClickedOn;
 	
-	switch ( eventInfo.itemRtti )
+	if ( ! qcanvasItem ) itemType = it_none;
+	else if ( dynamic_cast<Node*>(qcanvasItem) ) itemType = it_node;
+	else if ( dynamic_cast<ConnectorLine*>(qcanvasItem) || dynamic_cast<Connector*>(qcanvasItem) )
+		itemType = it_connector;
+	else if ( dynamic_cast<PinItem*>(qcanvasItem) ) itemType = it_pin;
+	else if ( dynamic_cast<ResizeHandle*>(qcanvasItem) ) itemType = it_resize_handle;
+	else if ( DrawPart *drawPartClickedOn = dynamic_cast<DrawPart*>(qcanvasItem) )
 	{
-		case ItemDocument::RTTI::None: itemType = CMManager::it_none; break;
-		case ItemDocument::RTTI::Node: itemType = CMManager::it_node; break;
-		case ItemDocument::RTTI::Connector: itemType = CMManager::it_connector; break;
-		case ItemDocument::RTTI::Pin: itemType = CMManager::it_pin; break;
-		case ItemDocument::RTTI::ResizeHandle: itemType = CMManager::it_resize_handle; break;
-		case ItemDocument::RTTI::DrawPart:
-		{
-			itemType = CMManager::it_drawpart;
-			DrawPart *drawPartClickedOn = dynamic_cast<DrawPart*>(eventInfo.qcanvasItemClickedOn);
+		itemType = it_drawpart;
 			
-			if ( drawPartClickedOn->mousePressEvent(eventInfo) )
-			{
-				p_lastItemClicked = drawPartClickedOn;
-				return;
-			}
+		if ( drawPartClickedOn->mousePressEvent(eventInfo) ) {
+			p_lastItemClicked = drawPartClickedOn;
+			return;
+		}
+
+		if ( drawPartClickedOn->isMovable() )
+			cnItemType |= CMManager::isi_isMovable;
+	} else if ( MechanicsItem *p_mechanicsItemClickedOn = dynamic_cast<MechanicsItem*>(qcanvasItem) )
+	{
+		itemType = it_mechanics_item;
 			
-			if ( drawPartClickedOn->isMovable() )
-				cnItemType |= CMManager::isi_isMovable;
-			break;
-		}
-		case ItemDocument::RTTI::Widget:
+		if ( p_mechanicsItemClickedOn->mousePressEvent(eventInfo) )
 		{
-			Widget *widget = dynamic_cast<Widget*>(eventInfo.qcanvasItemClickedOn);
-			if (widget)
-				eventInfo.qcanvasItemClickedOn = widget->parent();
+			p_lastItemClicked = p_mechanicsItemClickedOn;
+			return;
 		}
-		case ItemDocument::RTTI::CNItem:
+	} else {
+		if ( Widget *widget = dynamic_cast<Widget*>(qcanvasItem) )
+			qcanvasItem = widget->parent();
+		
+		if ( CNItem *cnItemClickedOn = dynamic_cast<CNItem*>(qcanvasItem) )
 		{
-			itemType = CMManager::it_canvas_item;
-			CNItem *cnItemClickedOn = dynamic_cast<CNItem*>(eventInfo.qcanvasItemClickedOn);
+			itemType = it_canvas_item;
 			
 			if ( cnItemClickedOn->mousePressEvent(eventInfo) )
 			{
@@ -149,19 +155,6 @@ void CMManager::mousePressEvent( EventInfo eventInfo )
 			
 			if ( cnItemClickedOn->isMovable() )
 				cnItemType |= CMManager::isi_isMovable;
-			break;
-		}
-		case ItemDocument::RTTI::MechanicsItem:
-		{
-			itemType = CMManager::it_mechanics_item;
-			MechanicsItem *p_mechanicsItemClickedOn = dynamic_cast<MechanicsItem*>(eventInfo.qcanvasItemClickedOn);
-			
-			if ( p_mechanicsItemClickedOn->mousePressEvent(eventInfo) )
-			{
-				p_lastItemClicked = p_mechanicsItemClickedOn;
-				return;
-			}
-			break;
 		}
 	}
 	
@@ -175,6 +168,7 @@ void CMManager::mousePressEvent( EventInfo eventInfo )
 			m_canvasManipulator = (*it)->m_createManipulatorPtr( p_itemDocument, this );
 		}
 	}
+
 	if (m_canvasManipulator)
 	{
 		if (m_canvasManipulator->mousePressedInitial(eventInfo))
@@ -188,12 +182,24 @@ void CMManager::mousePressEvent( EventInfo eventInfo )
 
 void CMManager::mouseDoubleClickEvent( const EventInfo &eventInfo )
 {
+	if (m_canvasManipulator)
+	{
+		// Translate this into a repeat-click event
+		if (m_canvasManipulator->mousePressedRepeat(eventInfo))
+		{
+			delete m_canvasManipulator;
+			m_canvasManipulator = 0l;
+		}
+		return;
+	}
+	
 	Item *item = dynamic_cast<Item*>(eventInfo.qcanvasItemClickedOn);
 	if (item)
 	{
 		item->mouseDoubleClickEvent(eventInfo);
 		return;
 	}
+
 	Widget *widget = dynamic_cast<Widget*>(eventInfo.qcanvasItemClickedOn);
 	if (widget)
 	{
@@ -209,11 +215,8 @@ void CMManager::mouseMoveEvent( const EventInfo &eventInfo )
 	{
 		if (m_canvasManipulator->mouseMoved(eventInfo))
 		{
-			kdDebug() << k_funcinfo << "About to delete" << endl;
 			delete m_canvasManipulator;
-			kdDebug() << k_funcinfo << "Deleted" << endl;
 			m_canvasManipulator = 0l;
-			kdDebug() << k_funcinfo << "Nullified" << endl;
 		}
 		ItemView *itemView = dynamic_cast<ItemView*>(p_itemDocument->activeView());
 		if (itemView)
@@ -225,31 +228,24 @@ void CMManager::mouseMoveEvent( const EventInfo &eventInfo )
 	QCanvasItem *qcnItem = p_itemDocument->itemAtTop(eventInfo.pos);
 	Item *item;
 	Widget *widget = dynamic_cast<Widget*>(qcnItem);
-	if (widget)
-		item = widget->parent();
-	else
-		item = dynamic_cast<Item*>(qcnItem);
+	if (widget) item = widget->parent();
+	else item = dynamic_cast<Item*>(qcnItem);
 	
-	if ( p_lastMouseOverItem != (QGuardedPtr<Item>)item )
-	{
+	if ( p_lastMouseOverItem != (QGuardedPtr<Item>)item ) {
 		QEvent event(QEvent::Leave);
 		
 		if (p_lastMouseOverItem)
 			p_lastMouseOverItem->leaveEvent();
 		
-		if (item)
-			item->enterEvent();
+		if (item) item->enterEvent();
 		
 		p_lastMouseOverItem = item;
 	}
 	
 	// If we clicked on an item, then continue to pass mouse events to that item until we release the mouse...
-	if (p_lastItemClicked)
-	{
+	if (p_lastItemClicked) {
 		p_lastItemClicked->mouseMoveEvent(eventInfo);
-	}
-	else if (item)
-	{
+	} else if (item) {
 		item->mouseMoveEvent(eventInfo);
 	}
 	//END
@@ -300,11 +296,8 @@ void CMManager::wheelEvent( const EventInfo &eventInfo )
 		Widget *widget = dynamic_cast<Widget*>(qcnItem);
 		if (widget)
 			item = widget->parent();
-		else
-			item = dynamic_cast<Item*>(qcnItem);
-		
-		if (item)
-			accepted = item->wheelEvent(eventInfo);
+		else	item = dynamic_cast<Item*>(qcnItem);
+		if (item) accepted = item->wheelEvent(eventInfo);
 	}
 	if (!accepted)
 	{
@@ -357,10 +350,11 @@ void CMManager::setRepeatedAddId( const QString & repeatedId )
 {
 	m_repeatedItemId = repeatedId;
 }
+//END class CMManager
 
 
 
-
+//BEGIN class CanvasManipulator
 CanvasManipulator::CanvasManipulator( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	p_itemDocument = itemDocument;
@@ -373,6 +367,8 @@ CanvasManipulator::CanvasManipulator( ItemDocument *itemDocument, CMManager *cmM
 	p_mechItemSelectList = dynamic_cast<MechanicsGroup*>(p_selectList);
 	p_cnItemClickedOn = 0l;
 	p_cmManager = cmManager;
+	
+	connect( itemDocument->canvas(), SIGNAL(resized( const QRect&, const QRect& )), this, SLOT(canvasResized( const QRect&, const QRect& )) );
 }
 
 
@@ -381,19 +377,30 @@ CanvasManipulator::~CanvasManipulator()
 }
 
 
+QPoint CanvasManipulator::snapPoint( QPoint point )
+{
+	point /= 8;
+	point *= 8;
+	point += QPoint( 4, 4 );
+	return point;
+}
+//END class CanvasManipulator
 
 
 CMRepeatedItemAdd::CMRepeatedItemAdd( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 }
+
 CMRepeatedItemAdd::~CMRepeatedItemAdd()
 {
 }
+
 CanvasManipulator* CMRepeatedItemAdd::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMRepeatedItemAdd(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMRepeatedItemAdd::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -401,6 +408,7 @@ ManipulatorInfo *CMRepeatedItemAdd::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMRepeatedItemAdd::construct;
 	return eventInfo;
 }
+
 bool CMRepeatedItemAdd::acceptManipulation( uint /*eventState*/, uint cmState, uint /*itemType*/, uint /*cnItemType*/ )
 {
 	return (cmState & CMManager::cms_repeated_add);
@@ -410,6 +418,7 @@ bool CMRepeatedItemAdd::mousePressedRepeat( const EventInfo &eventInfo )
 {
 	return mousePressedInitial(eventInfo);
 }
+
 bool CMRepeatedItemAdd::mousePressedInitial( const EventInfo &eventInfo )
 {
 	m_eventInfo = eventInfo;
@@ -437,19 +446,20 @@ bool CMRepeatedItemAdd::mouseReleased( const EventInfo &/*eventInfo*/ )
 }
 
 
-
-
 CMRightClick::CMRightClick( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 }
+
 CMRightClick::~CMRightClick()
 {
 }
+
 CanvasManipulator* CMRightClick::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMRightClick(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMRightClick::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -458,6 +468,7 @@ ManipulatorInfo *CMRightClick::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMRightClick::construct;
 	return eventInfo;
 }
+
 bool CMRightClick::acceptManipulation( uint eventState, uint /*cmState*/, uint /*itemType*/, uint /*cnItemType*/ )
 {
 	return eventState & CMManager::es_right_click;
@@ -507,15 +518,6 @@ QColor ConnectorDraw::validConnectionColor()
 }
 
 
-static double qpoint_distance( const QPoint & p1, const QPoint & p2 )
-{
-	double dx = p1.x() - p2.x();
-	double dy = p1.y() - p2.y();
-	
-	return std::sqrt( dx*dx + dy*dy );
-}
-
-
 QPoint ConnectorDraw::toValidPos( const QPoint & clickPos, Connector * clickedConnector ) const
 {
 	if ( !clickedConnector )
@@ -544,15 +546,7 @@ Connector * ConnectorDraw::toConnector( Node * node )
 	if ( !node || node->numCon( true, false ) < 3 )
 		return 0l;
 	
-	const ConnectorList inList = node->inputConnectorList();
-	if ( !inList.isEmpty() )
-		return *inList.begin();
-	
-	const ConnectorList outList = node->outputConnectorList();
-	if ( !outList.isEmpty() )
-		return *outList.begin();
-	
-	return 0l;
+	return node->getAConnector();
 }
 
 
@@ -583,7 +577,6 @@ void ConnectorDraw::grabEndStuff( QCanvasItem * endItem, const QPoint & pos, boo
 //END class ConnectorDraw
 
 
-
 //BEGIN class CMAutoConnector
 CMAutoConnector::CMAutoConnector( ItemDocument *itemDocument, CMManager *cmManager )
 	: ConnectorDraw( itemDocument, cmManager )
@@ -592,15 +585,18 @@ CMAutoConnector::CMAutoConnector( ItemDocument *itemDocument, CMManager *cmManag
 	p_startNode = 0l;
 	p_startConnector = 0l;
 }
+
 CMAutoConnector::~CMAutoConnector()
 {
 	delete m_connectorLine;
-	m_connectorLine = 0l;
+
 }
+
 CanvasManipulator* CMAutoConnector::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMAutoConnector(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMAutoConnector::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -608,6 +604,7 @@ ManipulatorInfo *CMAutoConnector::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMAutoConnector::construct;
 	return eventInfo;
 }
+
 bool CMAutoConnector::acceptManipulation( uint /*eventState*/, uint cmState, uint itemType, uint /*cnItemType*/ )
 {
 	return (itemType & (CMManager::it_node | CMManager::it_connector)) && !(cmState & CMManager::cms_manual_route);
@@ -627,15 +624,11 @@ bool CMAutoConnector::mousePressedInitial( const EventInfo &eventInfo )
 			p_startConnector = toConnector(p_startNode);
 			p_startNode = 0l;
 		}
-	}
-	else if (p_startConnector = dynamic_cast<Connector*>(eventInfo.qcanvasItemClickedOn) )
+	} else if (p_startConnector = dynamic_cast<Connector*>(eventInfo.qcanvasItemClickedOn) )
 	{
 // 		startConnectorPoint = m_eventInfo.pos = m_prevPos = p_icnDocument->gridSnap(m_eventInfo.pos);
 		startConnectorPoint = m_eventInfo.pos = m_prevPos = toValidPos( m_eventInfo.pos, p_startConnector );
-	}
-	
-	else
-		return true;
+	} else return true;
 		
 	p_icnDocument->unselectAll();
 	
@@ -652,7 +645,7 @@ bool CMAutoConnector::mouseMoved( const EventInfo &eventInfo )
 {
 	const QPoint pos = eventInfo.pos;
 	
-    int newX = p_icnDocument->gridSnap( pos.x() );
+	int newX = p_icnDocument->gridSnap( pos.x() );
 	int newY = p_icnDocument->gridSnap( pos.y() );
 
 	bool movedFlag = false;
@@ -671,8 +664,7 @@ bool CMAutoConnector::mouseMoved( const EventInfo &eventInfo )
 
 	m_connectorLine->setPoints( m_eventInfo.pos.x(), m_eventInfo.pos.y(), newX, newY );
 		
-	if (movedFlag)
-	{
+	if (movedFlag) {
 		QCanvasItem *startItem = 0l;
 		if (p_startNode)
 			startItem = p_startNode;
@@ -680,8 +672,8 @@ bool CMAutoConnector::mouseMoved( const EventInfo &eventInfo )
 			startItem = p_startConnector;
 		
 		QCanvasItem *endItem = p_icnDocument->itemAtTop( QPoint( newX, newY ) );
-		if ( endItem && endItem->rtti() == ItemDocument::RTTI::CNItem )
-			endItem = (static_cast<CNItem*>(endItem))->getClosestNode( QPoint( newX, newY ) );
+		if ( CNItem * cni = dynamic_cast<CNItem*>(endItem) )
+			endItem = cni->getClosestNode( QPoint( newX, newY ) );
 		
 		bool validLine = p_icnDocument->canConnect( startItem, endItem );
 		m_connectorLine->setPen( validLine ? validConnectionColor() : Qt::black );
@@ -706,36 +698,23 @@ bool CMAutoConnector::mouseReleased( const EventInfo &eventInfo )
 	
 	if (p_startConnector)
 	{
-		if (p_endConnector)
-		{
+		if (p_endConnector) {
 			if ( !p_icnDocument->createConnector( p_endConnector, p_startConnector, p_icnDocument->gridSnap(pos), startConnectorPoint ) )
 				return true;
-		}
-		else if (p_endNode)
-		{
+		} else if (p_endNode) {
 			if ( !p_icnDocument->createConnector( p_endNode, p_startConnector, startConnectorPoint ) )
 				return true;
-		}
-		else
-			return true;
-	}
-	else if (p_startNode)
-	{
+		} else	return true;
+	} else if (p_startNode) {
 		if (p_endConnector)
 		{
 			if ( !p_icnDocument->createConnector( p_startNode, p_endConnector, p_icnDocument->gridSnap(pos) ) )
 				return true;
-		}
-		else if (p_endNode)
-		{
+		} else if (p_endNode) {
 			if ( !p_icnDocument->createConnector( p_startNode, p_endNode ) )
 				return true;
-		}
-		else
-			return true;
-	}
-	else
-		return true;
+		} else	return true;
+	} else return true;
 	
 	p_itemDocument->requestStateSave();
 	return true;
@@ -750,15 +729,17 @@ CMManualConnector::CMManualConnector( ItemDocument *itemDocument, CMManager *cmM
 {
 	m_manualConnectorDraw = 0l;
 }
+
 CMManualConnector::~CMManualConnector()
 {
 	delete m_manualConnectorDraw;
-	m_manualConnectorDraw = 0l;
 }
+
 CanvasManipulator* CMManualConnector::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMManualConnector(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMManualConnector::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -766,6 +747,7 @@ ManipulatorInfo *CMManualConnector::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMManualConnector::construct;
 	return eventInfo;
 }
+
 bool CMManualConnector::acceptManipulation( uint /*eventState*/, uint cmState, uint itemType, uint /*cnItemType*/ )
 {
 	return (itemType & (CMManager::it_node | CMManager::it_connector)) && (cmState & CMManager::cms_manual_route);
@@ -774,19 +756,16 @@ bool CMManualConnector::acceptManipulation( uint /*eventState*/, uint cmState, u
 
 bool CMManualConnector::mousePressedInitial( const EventInfo &eventInfo )
 {
-	if ( eventInfo.isRightClick ) {
-		return true;
-	}
-	
+	if ( eventInfo.isRightClick ) return true;
+
 	m_eventInfo = eventInfo;
 	
 	p_icnDocument->unselectAll();
 	
 	QPoint sp;
 	
-	if ( eventInfo.itemRtti == ItemDocument::RTTI::Node )
+	if ( p_startNode = dynamic_cast<Node*>(eventInfo.qcanvasItemClickedOn) )
 	{
-		p_startNode = static_cast<Node*>(eventInfo.qcanvasItemClickedOn);
 		sp.setX( (int)p_startNode->x() );
 		sp.setY( (int)p_startNode->y() );
 		if ( p_startNode->numCon( true, false ) > 2 )
@@ -873,10 +852,8 @@ bool CMManualConnector::mouseMoved( const EventInfo &eventInfo )
 
 bool CMManualConnector::mouseReleased( const EventInfo &eventInfo )
 {
-	if (!m_manualConnectorDraw) {
-		return true;
-	}
-	
+	if (!m_manualConnectorDraw) return true;
+
 	QPoint pos = p_icnDocument->gridSnap(eventInfo.pos);
 	
 	grabEndStuff( m_manualConnectorDraw->mouseClicked(pos), pos, true );
@@ -895,28 +872,22 @@ bool CMManualConnector::mouseReleased( const EventInfo &eventInfo )
 		{
 			if ( !p_icnDocument->createConnector( p_endConnector, p_startConnector, p_icnDocument->gridSnap(pos), startConnectorPoint, &list ) )
 				return true;
-		}
-		else // if (p_endNode)
+		} else // if (p_endNode)
 		{
 			if ( !p_icnDocument->createConnector( p_endNode, p_startConnector, startConnectorPoint, &list ) )
 				return true;
 		}
-	}
-	else if (p_startNode)
-	{
+	} else if (p_startNode) {
 		if (p_endConnector)
 		{
 			if ( !p_icnDocument->createConnector( p_startNode, p_endConnector, p_icnDocument->gridSnap(pos), &list ) )
 				return true;
-		}
-		else // if (p_endNode)
+		} else // if (p_endNode)
 		{
 			if ( !p_icnDocument->createConnector( p_startNode, p_endNode, &list ) )
 				return true;
 		}
-	}
-	else
-		return true;
+	} else return true;
 	
 	p_itemDocument->requestStateSave();
 	return true;
@@ -924,20 +895,23 @@ bool CMManualConnector::mouseReleased( const EventInfo &eventInfo )
 //END class CMManualConnector
 
 
-
-
+//BEGIN class CMItemMove
 CMItemMove::CMItemMove( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 	p_flowContainerCandidate = 0l;
+	m_bItemsSnapToGrid = false;
 }
+
 CMItemMove::~CMItemMove()
 {
 }
+
 CanvasManipulator* CMItemMove::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMItemMove(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMItemMove::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -945,6 +919,7 @@ ManipulatorInfo *CMItemMove::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMItemMove::construct;
 	return eventInfo;
 }
+
 bool CMItemMove::acceptManipulation( uint eventState, uint /*cmState*/, uint itemType, uint cnItemType )
 {
 	return ((itemType & CMManager::it_canvas_item) || (itemType & CMManager::it_drawpart)) && (cnItemType & CMManager::isi_isMovable) && !(eventState & CMManager::es_right_click);
@@ -957,9 +932,8 @@ bool CMItemMove::mousePressedInitial( const EventInfo &eventInfo )
 	m_prevPos = eventInfo.pos;
 	
 	Item *item = dynamic_cast<Item*>(eventInfo.qcanvasItemClickedOn);
-	if (!item)
-		return true;
-	
+
+	if (!item) return true;
 	
 	if ( !p_selectList->contains(item) )
 	{
@@ -967,8 +941,7 @@ bool CMItemMove::mousePressedInitial( const EventInfo &eventInfo )
 			p_itemDocument->unselectAll();
 		
 		p_itemDocument->select(item);
-	}
-	else if (m_eventInfo.ctrlPressed)
+	} else if (m_eventInfo.ctrlPressed)
 		p_itemDocument->unselect(item);
 	
 	if ( p_selectList->isEmpty() )
@@ -989,6 +962,7 @@ bool CMItemMove::mousePressedInitial( const EventInfo &eventInfo )
 	ItemList itemList = p_cnItemSelectList->items(false);
 	itemList.remove((Item*)0l);
 	
+	m_bItemsSnapToGrid = false;
 	const ItemList::iterator itemListEnd = itemList.end();
 	for ( ItemList::iterator it = itemList.begin(); it != itemListEnd; ++it )
 	{
@@ -996,9 +970,13 @@ bool CMItemMove::mousePressedInitial( const EventInfo &eventInfo )
 		if ( !cnItem || !cnItem->canvas() )
 			continue;
 			
-		cnItem->setInitialPos(m_eventInfo.pos);
+		m_bItemsSnapToGrid = true;
 	}
 	
+	if ( m_bItemsSnapToGrid )
+		m_prevSnapPoint = this->snapPoint( m_prevPos );
+	else	m_prevSnapPoint = m_prevPos;
+	
 	ConnectorList fixedConnectors;
 	p_icnDocument->getTranslatable( itemList, &fixedConnectors, &m_translatableConnectors, &m_translatableNodeGroups );
 	
@@ -1012,36 +990,74 @@ bool CMItemMove::mousePressedInitial( const EventInfo &eventInfo )
 }
 
 
+void CMItemMove::canvasResized( const QRect & oldSize, const QRect & newSize )
+{
+	QPoint delta = oldSize.topLeft() - newSize.topLeft();
+	
+// 	scrollCanvasToSelection();
+	
+// 	QCursor::setPos( QCursor::pos() + delta );
+// 	m_prevPos += delta;
+// 	m_prevSnapPoint += delta;
+}
+
+
+void CMItemMove::scrollCanvasToSelection()
+{
+	QRect bound;
+	ItemList itemList = p_cnItemSelectList->items(false);
+	itemList.remove((Item*)0l);
+	const ItemList::iterator itemListEnd = itemList.end();
+	for ( ItemList::iterator it = itemList.begin(); it != itemListEnd; ++it )
+		bound |= (*it)->boundingRect();
+	
+	QPoint scrollToPos = m_prevPos;
+	if ( m_dx < 0 ) {
+		// Scrolling left
+		scrollToPos -= QPoint( bound.left(), 0 );
+	} else {
+		// Scrolling right
+		scrollToPos += QPoint( bound.right(), 0 );
+	}
+
+	if ( m_dy < 0 ) {
+		// Scrolling up
+		scrollToPos -= QPoint( 0, bound.top() );
+	} else {
+		// Scrolling right
+		scrollToPos += QPoint( 0, bound.bottom() );
+	}
+
+	ItemView *itemView = dynamic_cast<ItemView*>(p_itemDocument->activeView());
+	if (itemView)
+		itemView->scrollToMouse( scrollToPos );
+}
+
+
 bool CMItemMove::mouseMoved( const EventInfo &eventInfo )
 {
-	const QPoint pos = eventInfo.pos;
+	QPoint pos = eventInfo.pos;
 	
-	int x = pos.x();
-	int y = pos.y();
+	QPoint snapPoint = pos;
+	if ( m_bItemsSnapToGrid )
+		snapPoint = this->snapPoint( snapPoint );
 	
+	int dx = snapPoint.x() - m_prevSnapPoint.x();
+	int dy = snapPoint.y() - m_prevSnapPoint.y();
+	
+	m_dx = dx;
+	m_dy = dy;
 	
    	const ItemList itemList = p_cnItemSelectList->items();
 	const ItemList::const_iterator end = itemList.end();
-	int dx=0, dy=0;
+	
 	for ( ItemList::const_iterator it = itemList.begin(); it != end; ++it )
 	{
 		if ( !*it || !(*it)->isMovable() )
 			continue;
 		
-		const QRect oldRect = (*it)->boundingRect();
-		(*it)->setChanged();
-		
-		if ( CNItem *cnItem = dynamic_cast<CNItem*>((Item*)*it) )
-		{
-			dx = -int((*it)->x());
-			dy = -int((*it)->y());
-			cnItem->snap( x, y );
-			dx += int((*it)->x());
-			dy += int((*it)->y());
-		}
-		else
-			(*it)->moveBy( eventInfo.pos.x()-m_prevPos.x(), eventInfo.pos.y()-m_prevPos.y() );
-		
+		QRect oldRect = (*it)->boundingRect();
+		(*it)->moveBy( dx, dy );
 		QRect newRect = (*it)->boundingRect();
 		QRect merged = oldRect | newRect;
 	}
@@ -1066,6 +1082,7 @@ bool CMItemMove::mouseMoved( const EventInfo &eventInfo )
 			p_flowContainerCandidate = 0l;
 		}
 	}
+
 	if (fc)
 	{
 		p_flowContainerCandidate = fc;
@@ -1074,21 +1091,28 @@ bool CMItemMove::mouseMoved( const EventInfo &eventInfo )
 	
 	p_itemDocument->requestEvent( ItemDocument::ItemDocumentEvent::ResizeCanvasToItems );
 	p_canvas->update();
-	m_prevPos = eventInfo.pos;
+	m_prevPos = pos;
+	m_prevSnapPoint = snapPoint;
+	
+// 	scrollCanvasToSelection();
+	
 	return false;
 }
 
 
 bool CMItemMove::mouseReleased( const EventInfo &eventInfo )
 {
+	// Is the release event from a right click (which rotates items)?
+	if ( eventInfo.isRightClick || eventInfo.isMiddleClick )
+		return false;
+	
 	QStringList itemIDs;
 	
 	const ItemList itemList = p_cnItemSelectList->items();
 	const ItemList::const_iterator ilEnd = itemList.end();
 	for ( ItemList::const_iterator it = itemList.begin(); it != ilEnd; ++it )
 	{
-		if (*it)
-			itemIDs.append( (*it)->id() );
+		if (*it) itemIDs.append( (*it)->id() );
 	}
 	
 	const QPoint pos = eventInfo.pos;
@@ -1108,9 +1132,7 @@ bool CMItemMove::mouseReleased( const EventInfo &eventInfo )
 		
 		p_flowContainerCandidate->setSelected(false);
 		p_flowContainerCandidate = 0l;
-	}
-	else
-	{
+	} else {
 		for ( ItemList::const_iterator it = itemList.begin(); it != ilEnd; ++it )
 			(*it)->setParentItem(0l);
 	}
@@ -1124,11 +1146,9 @@ bool CMItemMove::mouseReleased( const EventInfo &eventInfo )
 			flowContainer->setFullBounds(false);
 	}
 	
-	if (p_icnDocument)
-		p_icnDocument->requestRerouteInvalidatedConnectors();
+	if (p_icnDocument) p_icnDocument->requestRerouteInvalidatedConnectors();
 	
-	if ( m_eventInfo.pos != eventInfo.pos )
-		p_itemDocument->requestStateSave();
+	if ( m_eventInfo.pos != eventInfo.pos ) p_itemDocument->requestStateSave();
 	
 	p_itemDocument->requestEvent( ItemDocument::ItemDocumentEvent::ResizeCanvasToItems );
 	
@@ -1136,20 +1156,32 @@ bool CMItemMove::mouseReleased( const EventInfo &eventInfo )
 }
 
 
-
+bool CMItemMove::mousePressedRepeat( const EventInfo & info )
+{
+	if ( info.isRightClick )
+		p_cnItemSelectList->slotRotateCW();
+	else if ( info.isMiddleClick )
+		p_cnItemSelectList->flipHorizontally();
+	
+	return false;
+}
+//END class CMItemMove
 
 
 CMItemResize::CMItemResize( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 }
+
 CMItemResize::~CMItemResize()
 {
 }
+
 CanvasManipulator* CMItemResize::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMItemResize(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMItemResize::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -1158,13 +1190,13 @@ ManipulatorInfo *CMItemResize::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMItemResize::construct;
 	return eventInfo;
 }
+
 bool CMItemResize::acceptManipulation( uint eventState, uint /*cmState*/, uint itemType, uint /*cnItemType*/ )
 {
 	return (itemType & CMManager::it_resize_handle) && !(eventState & CMManager::es_right_click);
 }
 
 
-
 bool CMItemResize::mousePressedInitial( const EventInfo &eventInfo )
 {
 	m_eventInfo = eventInfo;
@@ -1183,8 +1215,8 @@ bool CMItemResize::mouseMoved( const EventInfo &eventInfo )
 	// Shift pressed == snap to grid
 	if ( eventInfo.shiftPressed )
 	{
-		_x = int(_x/8)*8+4;
-		_y = int(_y/8)*8+4;
+		_x = snapToCanvas(_x);
+		_y = snapToCanvas(_y);
 	}
 	
 	p_resizeHandle->moveRH( _x, _y );
@@ -1201,22 +1233,20 @@ bool CMItemResize::mouseReleased( const EventInfo &/*eventInfo*/ )
 }
 
 
-
-
-
-
-
 CMMechItemMove::CMMechItemMove( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 }
+
 CMMechItemMove::~CMMechItemMove()
 {
 }
+
 CanvasManipulator* CMMechItemMove::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMMechItemMove(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMMechItemMove::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -1225,6 +1255,7 @@ ManipulatorInfo *CMMechItemMove::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMMechItemMove::construct;
 	return eventInfo;
 }
+
 bool CMMechItemMove::acceptManipulation( uint eventState, uint /*cmState*/, uint itemType, uint /*cnItemType*/ )
 {
 	return ((itemType & CMManager::it_mechanics_item) || (itemType & CMManager::it_drawpart)) && !(eventState & CMManager::es_right_click);
@@ -1242,8 +1273,7 @@ bool CMMechItemMove::mousePressedInitial( const EventInfo &eventInfo )
 	
 	MechanicsItem *mechItem = dynamic_cast<MechanicsItem*>(eventInfo.qcanvasItemClickedOn);
 	
-	if (mechItem)
-		m_prevClickedOnSM = mechItem->selectionMode();
+	if (mechItem) m_prevClickedOnSM = mechItem->selectionMode();
 	
 	if (eventInfo.shiftPressed)
 	{
@@ -1254,8 +1284,7 @@ bool CMMechItemMove::mousePressedInitial( const EventInfo &eventInfo )
 			mechItem->setSelectionMode(MechanicsItem::sm_move);
 			mechItem->setParentItem(0l);
 		}
-	}
-	else if ( !p_selectList->contains(mechItem) )
+	} else if ( !p_selectList->contains(mechItem) )
 	{
 		if (!eventInfo.ctrlPressed)
 			p_mechanicsDocument->unselectAll();
@@ -1264,9 +1293,7 @@ bool CMMechItemMove::mousePressedInitial( const EventInfo &eventInfo )
 		
 		if (mechItem)
 			mechItem->setSelectionMode(MechanicsItem::sm_move);
-	}
-	else
-	{
+	} else {
 		if (mechItem)
 			mechItem->setSelectionMode(MechanicsItem::sm_move);
 		
@@ -1411,15 +1438,17 @@ CMSelect::CMSelect( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	m_selectRectangle = 0l;
 }
+
 CMSelect::~CMSelect()
 {
 	delete m_selectRectangle;
-	m_selectRectangle = 0l;
 }
+
 CanvasManipulator* CMSelect::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMSelect(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMSelect::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -1428,6 +1457,7 @@ ManipulatorInfo *CMSelect::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMSelect::construct;
 	return eventInfo;
 }
+
 bool CMSelect::acceptManipulation( uint /*eventState*/, uint /*cmState*/, uint itemType, uint /*cnItemType*/ )
 {
 	return (itemType & CMManager::it_none);
@@ -1476,20 +1506,21 @@ bool CMSelect::mouseReleased( const EventInfo &/*eventInfo*/ )
 //END class CMSelect
 
 
-
-
 CMItemDrag::CMItemDrag( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
 {
 	b_dragged = false;
 }
+
 CMItemDrag::~CMItemDrag()
 {
 }
+
 CanvasManipulator* CMItemDrag::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMItemDrag(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMItemDrag::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -1498,6 +1529,7 @@ ManipulatorInfo *CMItemDrag::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMItemDrag::construct;
 	return eventInfo;
 }
+
 bool CMItemDrag::acceptManipulation( uint /*eventState*/, uint /*cmState*/, uint itemType, uint cnItemType )
 {
 	return (itemType & (CMManager::it_canvas_item|CMManager::it_pin)) && !(cnItemType & CMManager::isi_isMovable);
@@ -1525,8 +1557,8 @@ bool CMItemDrag::mouseMoved( const EventInfo &eventInfo )
 	
 		b_dragged = true;
 		
-		if ( m_eventInfo.itemRtti == ItemDocument::RTTI::Pin )
-			((PinItem*)m_eventInfo.qcanvasItemClickedOn)->dragged( pos.x() - m_eventInfo.pos.x() );
+		if ( PinItem * pi = dynamic_cast<PinItem*>(m_eventInfo.qcanvasItemClickedOn) )
+			pi->dragged( pos.x() - m_eventInfo.pos.x() );
 	}
 	return false;
 }
@@ -1534,8 +1566,11 @@ bool CMItemDrag::mouseMoved( const EventInfo &eventInfo )
 
 bool CMItemDrag::mouseReleased( const EventInfo &/*eventInfo*/ )
 {
-	if ( !b_dragged && m_eventInfo.itemRtti == ItemDocument::RTTI::Pin )
-		(static_cast<PinItem*>(m_eventInfo.qcanvasItemClickedOn))->switchState();
+	if ( !b_dragged )
+	{
+		if ( PinItem * pi = dynamic_cast<PinItem*>(m_eventInfo.qcanvasItemClickedOn) )
+			pi->switchState();
+	}
 	
 	p_itemDocument->requestStateSave();
 	return true;
@@ -1549,7 +1584,6 @@ CanvasEllipseDraw::CanvasEllipseDraw( int x, int y, QCanvas * canvas )
 	move( x, y );
 }
 
-
 void CanvasEllipseDraw::drawShape( QPainter & p )
 {
 	p.drawEllipse( int(x()-width()/2), int(y()-height()/2), width(), height() );
@@ -1557,7 +1591,6 @@ void CanvasEllipseDraw::drawShape( QPainter & p )
 //END class CanvasEllipseDraw
 
 
-
 //BEGIN class CMDraw
 CMDraw::CMDraw( ItemDocument *itemDocument, CMManager *cmManager )
 	: CanvasManipulator( itemDocument, cmManager )
@@ -1566,14 +1599,17 @@ CMDraw::CMDraw( ItemDocument *itemDocument, CMManager *cmManager )
 	m_pDrawRectangle = 0l;
 	m_pDrawEllipse = 0l;
 }
+
 CMDraw::~CMDraw()
 {
 	p_cmManager->setDrawAction(-1);
 }
+
 CanvasManipulator* CMDraw::construct( ItemDocument *itemDocument, CMManager *cmManager )
 {
 	return new CMDraw(itemDocument,cmManager);
 }
+
 ManipulatorInfo *CMDraw::manipulatorInfo()
 {
 	ManipulatorInfo *eventInfo = new ManipulatorInfo();
@@ -1581,12 +1617,12 @@ ManipulatorInfo *CMDraw::manipulatorInfo()
 	eventInfo->m_createManipulatorPtr = CMDraw::construct;
 	return eventInfo;
 }
+
 bool CMDraw::acceptManipulation( uint /*eventState*/, uint cmState, uint /*itemType*/, uint /*cnItemType*/ )
 {
 	return (cmState & CMManager::cms_draw);
 }
 
-
 bool CMDraw::mousePressedInitial( const EventInfo &eventInfo )
 {
 	m_eventInfo = eventInfo;
@@ -1595,10 +1631,11 @@ bool CMDraw::mousePressedInitial( const EventInfo &eventInfo )
 	{
 		case DrawPart::da_text:
 		case DrawPart::da_rectangle:
+		case DrawPart::da_image:
 		{
 			m_pDrawRectangle = new QCanvasRectangle( eventInfo.pos.x(), eventInfo.pos.y(), 0, 0, p_canvas );
 			m_pDrawRectangle->setPen( QPen( QColor(0,0,0), 1 ) );
-			m_pDrawRectangle->setZ( ICNDocument::Z::Select );
+			m_pDrawRectangle->setZ( ICNDocument::Z::ConnectorCreateLine );
 			m_pDrawRectangle->show();
 			break;
 		}
@@ -1606,7 +1643,7 @@ bool CMDraw::mousePressedInitial( const EventInfo &eventInfo )
 		{
 			m_pDrawEllipse = new CanvasEllipseDraw( eventInfo.pos.x(), eventInfo.pos.y(), p_canvas );
 			m_pDrawEllipse->setPen( QPen( QColor(0,0,0), 1 ) );
-			m_pDrawEllipse->setZ( ICNDocument::Z::Select );
+			m_pDrawEllipse->setZ( ICNDocument::Z::ConnectorCreateLine );
 			m_pDrawEllipse->show();
 			break;
 		}
@@ -1616,7 +1653,7 @@ bool CMDraw::mousePressedInitial( const EventInfo &eventInfo )
 			m_pDrawLine = new QCanvasLine(p_canvas);
 			m_pDrawLine->setPoints( eventInfo.pos.x(), eventInfo.pos.y(), eventInfo.pos.x(), eventInfo.pos.y() );
 			m_pDrawLine->setPen( QPen( QColor(0,0,0), 1 ) );
-			m_pDrawLine->setZ( ICNDocument::Z::Select );
+			m_pDrawLine->setZ( ICNDocument::Z::ConnectorCreateLine );
 			m_pDrawLine->show();
 			break;
 		}
@@ -1635,22 +1672,19 @@ bool CMDraw::mouseMoved( const EventInfo &eventInfo )
 	if (m_pDrawRectangle)
 		m_pDrawRectangle->setSize( pos.x()-m_eventInfo.pos.x(), pos.y()-m_eventInfo.pos.y() );
 	
-	else if (m_pDrawEllipse)
-	{
+	else if (m_pDrawEllipse) {
 // 		QRect r( m_eventInfo.pos.x(), m_eventInfo.pos.y(), pos.x()-m_eventInfo.pos.x(), pos.y()-m_eventInfo.pos.y() );
 // 		r = r.normalize();
 // 		
 // 		m_pDrawEllipse->setSize( r.width(), r.height() );
 // 		m_pDrawEllipse->move( r.left()+(r.width()/2), r.top()+(r.height()/2) );
 		
-		m_pDrawEllipse->setSize( 2*QABS(pos.x()-m_eventInfo.pos.x()), 2*QABS(pos.y()-m_eventInfo.pos.y()) );
+		m_pDrawEllipse->setSize( 2 * abs(pos.x() - m_eventInfo.pos.x()), 2 * abs(pos.y() - m_eventInfo.pos.y()) );
 	}
 	
 	else if (m_pDrawLine)
 		m_pDrawLine->setPoints( eventInfo.pos.x(), eventInfo.pos.y(), m_pDrawLine->endPoint().x(), m_pDrawLine->endPoint().y() );
-	
-	else
-		return true;
+	else return true;
 	
 	return false;
 }
@@ -1680,9 +1714,7 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 			
 			if ( m_pDrawRectangle->rect().height() < 0 )
 				sizeRect.moveTop( sizeRect.top() + 1);
-		}
-		else
-		{
+		} else {
 			int w = m_pDrawEllipse->width()+1;
 			int h = m_pDrawEllipse->height()+1;
 			int x = int(m_pDrawEllipse->x()-w/2);
@@ -1694,9 +1726,7 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 		delete m_pDrawEllipse;
 		m_pDrawRectangle = 0l;
 		m_pDrawEllipse = 0l;
-	}
-	else if (m_pDrawLine)
-	{
+	} else if (m_pDrawLine) {
 		int sx = m_pDrawLine->startPoint().x();
 		int sy = m_pDrawLine->startPoint().y();
 		int ex = m_pDrawLine->endPoint().x();
@@ -1706,9 +1736,7 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 		
 		delete m_pDrawLine;
 		m_pDrawLine = 0l;
-	}
-	else
-		return true;
+	} else return true;
 	
 	QString id;
 	switch ( (DrawPart::DrawAction) p_cmManager->drawAction() )
@@ -1717,6 +1745,10 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 			id = "dp/rectangle";
 			break;
 			
+		case DrawPart::da_image:
+			id = "dp/image";
+			break;
+			
 		case DrawPart::da_ellipse:
 			id = "dp/ellipse";
 			break;
@@ -1740,12 +1772,13 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 			id = "dp/arrow";
 			break;
 	}
-	if ( id.isEmpty() )
-		return true;
-	
-	Item * item = p_itemDocument->addItem( id, sizeRect.topLeft(), true );
-	if (!item)
-		return true;
+
+	if ( id.isEmpty() ) return true;
+
+	Item *item = p_itemDocument->addItem( id, sizeRect.topLeft(), true );
+
+	if (!item) return true;
+
 	item->move( sizeRect.x(), sizeRect.y() ); // We call this again as p_itemDocument->addItem will move the item if it is slightly off the canvas.
 	
 	item->setSize( 0, 0, sizeRect.width(), sizeRect.height() );
@@ -1756,7 +1789,6 @@ bool CMDraw::mouseReleased( const EventInfo &eventInfo )
 //END class CMDraw
 
 
-
 //BEGIN class ManualConnectorDraw
 ManualConnectorDraw::ManualConnectorDraw( ICNDocument *_icnDocument, const QPoint &initialPos )
 {
@@ -1794,7 +1826,6 @@ ManualConnectorDraw::~ManualConnectorDraw()
 	m_connectorLines.clear();
 }
 
-
 void ManualConnectorDraw::setColor( const QColor & color )
 {
 	m_color = color;
@@ -1804,7 +1835,6 @@ void ManualConnectorDraw::setColor( const QColor & color )
 		(*it)->setPen( m_color );
 }
 
-
 void ManualConnectorDraw::mouseMoved( const QPoint &pos )
 {
 	if ( m_currentPos == pos ) return;
@@ -1833,9 +1863,7 @@ QCanvasItem* ManualConnectorDraw::mouseClicked( const QPoint &pos )
 {
 	if (b_orientationDefined)
 		b_currentVertical = !b_currentVertical;
-	
-	else
-		mouseMoved(pos);
+	else	mouseMoved(pos);
 	
 	b_orientationDefined = true;
 
@@ -1867,9 +1895,7 @@ void ManualConnectorDraw::updateConnectorEnds()
 	{
 		pivot.setY( previousStart.y() );
 		m_currentCon->setPoints( pivot.x(), pivot.y(), pivot.x(), m_currentPos.y() );
-	}
-	else
-	{
+	} else {
 		pivot.setX( previousStart.x() );
 		m_currentCon->setPoints( pivot.x(), pivot.y(), m_currentPos.x(), pivot.y() );
 	}
