@@ -13,7 +13,7 @@
 
 #include "eventinfo.h"
 
-#include <qcanvas.h>
+#include <canvas.h>
 #include <qguardedptr.h>
 
 class CanvasManipulator;
@@ -92,9 +92,10 @@ public:
 		it_resize_handle =	1 << 6,
 		it_drawpart =		1 << 7
 	};
+
 	enum ItemStateInfo
 	{
-		isi_isMovable = 0x2,
+		isi_isMovable = 0x2
 	};
 	CMManager( ItemDocument *itemDocument );
 	~CMManager();
@@ -168,8 +169,10 @@ private slots:
 Abstract class for a "editing operation" on the ICNDocument, such as moving an item or resizing the canvas
 @author David Saxton
 */
-class CanvasManipulator
+class CanvasManipulator : public QObject
 {
+	Q_OBJECT
+	
 public:
 	CanvasManipulator( ItemDocument *itemDocument, CMManager *cmManager );
 	virtual ~CanvasManipulator();
@@ -209,6 +212,16 @@ public:
 	 * @returns true if the manipulation operation has finished
 	 */
 	virtual bool mouseReleased( const EventInfo &/*info*/ ) { return true; }
+	/**
+	 * Snaps the point to the 8-sized canvas grid.
+	 */
+	static QPoint snapPoint( QPoint point );
+	
+protected slots:
+	/**
+	 * Called when the working canvas emits a resized signal.
+	 */
+	virtual void canvasResized( const QRect & oldSize, const QRect & newSize ) { (void)oldSize; (void)newSize; }
 	
 protected:
 	Type m_type;
@@ -381,8 +394,16 @@ public:
 	virtual bool mousePressedInitial( const EventInfo &info );
 	virtual bool mouseMoved( const EventInfo &info );
 	virtual bool mouseReleased( const EventInfo &info );
+	virtual bool mousePressedRepeat( const EventInfo & info );
 	
 protected:
+	virtual void canvasResized( const QRect & oldSize, const QRect & newSize );
+	void scrollCanvasToSelection();
+	
+	QPoint m_prevSnapPoint;
+	bool m_bItemsSnapToGrid; ///< true iff selection contains CNItems
+	int m_dx;
+	int m_dy;
 	ConnectorList m_translatableConnectors;
 	NodeGroupList m_translatableNodeGroups;
 	FlowContainer *p_flowContainerCandidate;
