@@ -21,7 +21,7 @@
 #include "processchain.h"
 #include "simulator.h"
 
-#include <assert.h>
+#include <cassert>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -46,12 +46,11 @@ bool bUseGUI = true;
 // extern "C" void initialize_gpsim();
 // void initialize_gpsim(void);
 extern void initialize_commands();
-void initialize_ConsoleUI();
 extern void initialize_readline();
 extern void gui_main(void);
 extern void cli_main();
-void gpsim_version() {};
-void quit_gui() {};
+void gpsim_version() {}
+void quit_gui() {}
 
 
 //BEGIN class GpsimProcessor
@@ -77,9 +76,6 @@ GpsimProcessor::GpsimProcessor( QString symbolFile, QObject *parent )
 {
 	if (!bDoneGpsimInit)
 	{
-#ifndef GPSIM_0_21_4
-		initialize_ConsoleUI();
-#endif
 		initialize_gpsim_core();
 		initialization_is_complete();
 		
@@ -245,16 +241,23 @@ void GpsimProcessor::executeNext()
 	
 	unsigned long long beforeExecuteCount = get_cycles().get();
 	
-	m_pPicProcessor->step_one(false); // Don't know what the false is for; gpsim ignores its value anyway
-	
-	// Some instructions take more than one cycle to execute, so ignore next cycle if this was the case
-	if ( (get_cycles().get() - beforeExecuteCount) > 1 )
-		m_bCanExecuteNextCycle = false;
+	if(get_bp().have_interrupt())
+	{
+		m_pPicProcessor->interrupt();
+	}
+	else
+	{
+		m_pPicProcessor->step_one(false); // Don't know what the false is for; gpsim ignores its value anyway
+ 
+		// Some instructions take more than one cycle to execute, so ignore next cycle if this was the case
+		if ( (get_cycles().get() - beforeExecuteCount) > 1 )
+			m_bCanExecuteNextCycle = false;
+	}
 	
 	currentDebugger()->checkForBreak();
 	
-	// Let's also update the values of RegisterInfo every 50 milliseconds
-	if ( (beforeExecuteCount % 20000) == 0 )
+	// Let's also update the values of RegisterInfo every 25 milliseconds
+	if ( (beforeExecuteCount % 10000) == 0 )
 		registerMemory()->update();
 }
 
@@ -274,8 +277,10 @@ void GpsimProcessor::reset()
 
 MicroInfo * GpsimProcessor::microInfo( ) const
 {
-	if ( !m_pPicProcessor ) 
+	if ( !m_pPicProcessor ){
+		kdWarning() << k_funcinfo << " m_pPicProcessor == NULL" << endl;
 		return 0l;
+	}
 	
 	return MicroLibrary::self()->microInfoWithID( m_pPicProcessor->name().c_str() );
 }
@@ -728,6 +733,7 @@ void GpsimDebugger::stackStep( int dl )
 RegisterSet::RegisterSet( pic_processor * picProcessor )
 {
 	unsigned numRegisters = picProcessor->rma.get_size();
+	kdDebug() << k_funcinfo << "numRegisters="<<numRegisters<<endl;
 	m_registers.resize( numRegisters, 0l );
 	for ( unsigned i = 0; i < numRegisters; ++i )
 	{
@@ -810,7 +816,7 @@ RegisterInfo::RegisterInfo( Register * reg )
 			break;
 	}
 		
-	m_name = m_pRegister->baseName();
+	m_name = m_pRegister->baseName().c_str();
 }
 
 
