@@ -14,6 +14,7 @@
 #include "flowcode.h"
 #include "flowpart.h"
 #include "fpnode.h"
+#include "inputflownode.h"
 #include "itemdocument.h"
 #include "itemdocumentdata.h"
 #include "microsettings.h"
@@ -30,52 +31,58 @@
 #include <qpixmap.h>
 #include <qregexp.h>
 
-#include <assert.h>
+#include <cassert>
 #include <algorithm>
 #include <cmath>
 
+// Degrees per radian
+const double DPR = 57.29577951308232087665461840231273527024;
+
 // The following arrays of numbers represent the positions of nodes in different configurations,
 // with the numbers as NodeInfo::Position.
 
-Node::node_dir diamondNodePositioning[8][3] = {
-	{Node::dir_up,	Node::dir_down,	Node::dir_right},
-	{Node::dir_up,	Node::dir_down,	Node::dir_left},
-	{Node::dir_up,	Node::dir_right,Node::dir_down},
-	{Node::dir_up,	Node::dir_right,Node::dir_left},
-	{Node::dir_left,Node::dir_right,Node::dir_down},
-	{Node::dir_left,Node::dir_right,Node::dir_up},
-	{Node::dir_left,Node::dir_down,	Node::dir_right},
-	{Node::dir_left,Node::dir_down,	Node::dir_up} };
+int diamondNodePositioning[8][3] = {
+	{270,	90,	0},
+	{270,	90,	180},
+	{270,	0,	90},
+	{270,	0,	180},
+	{180,	0,	90},
+	{180,	0,	270},
+	{180,	90,	0},
+	{180,	90,	270} };
 
-Node::node_dir inOutNodePositioning[8][2] = {
-	{Node::dir_up,Node::dir_down},
-	{Node::dir_up,Node::dir_right},
-	{Node::dir_up,Node::dir_left},
-	{Node::dir_right,Node::dir_right}, // (invalid)
-	{Node::dir_left,Node::dir_right},
-	{Node::dir_left,Node::dir_down},
-	{Node::dir_left,Node::dir_up},
-	{Node::dir_right,Node::dir_right} }; // (invalid)
+	int inOutNodePositioning[8][2] = {
+	{270,	90},
+	{270,	0},
+	{270,	180},
+	{0,		0}, // (invalid)
+	{180,	0},
+	{180,	90},
+	{180,	270},
+	{0,		0} }; // (invalid)
 
-Node::node_dir inNodePositioning[4] = {Node::dir_up,Node::dir_right,Node::dir_down,Node::dir_left};
+int inNodePositioning[4] = {270,0,90,180};
 
-Node::node_dir outNodePositioning[4] = {Node::dir_down,Node::dir_left,Node::dir_up,Node::dir_right};
+int outNodePositioning[4] = {90,180,270,0};
 
 FlowPart::FlowPart( ICNDocument *icnDocument, bool newItem, const QString &id )
 	: CNItem( icnDocument, newItem, id )
 {
-	icnDocument->registerItem(this);
-	m_pFlowCodeDocument = dynamic_cast<FlowCodeDocument*>(icnDocument);
-	assert( m_pFlowCodeDocument );
-	
 	m_flowSymbol = FlowPart::ps_other;
 	m_orientation = 0;
 	m_stdInput = 0l;
 	m_stdOutput = 0l;
 	m_altOutput = 0l;
 	
-	connect( m_pFlowCodeDocument, SIGNAL(picTypeChanged()), this, SLOT(slotUpdateFlowPartVariables()) );
-	connect( m_pFlowCodeDocument, SIGNAL(pinMappingsChanged()), this, SLOT(slotUpdateFlowPartVariables()) );
+	if ( icnDocument )
+	{
+		icnDocument->registerItem(this);
+		m_pFlowCodeDocument = dynamic_cast<FlowCodeDocument*>(icnDocument);
+		assert( m_pFlowCodeDocument );
+	
+		connect( m_pFlowCodeDocument, SIGNAL(picTypeChanged()), this, SLOT(slotUpdateFlowPartVariables()) );
+		connect( m_pFlowCodeDocument, SIGNAL(pinMappingsChanged()), this, SLOT(slotUpdateFlowPartVariables()) );
+	}
 }
 
 
@@ -115,27 +122,19 @@ void FlowPart::setCaption( const QString &caption )
 	switch(m_flowSymbol)
 	{
 		case FlowPart::ps_call:
-		{
 			width += 48;
 			break;
-		}
 		case FlowPart::ps_io:
 		case FlowPart::ps_round:
-		{
 			width += 32;
 			break;
-		}
 		case FlowPart::ps_decision:
-		{
 			width += 64;
 			break;
-		}
 		case FlowPart::ps_process:
 		default:
-		{
 			width += 32;
 			break;
-		}
 	}
 	
 	bool hasSideConnectors = m_flowSymbol == FlowPart::ps_decision;
@@ -153,17 +152,17 @@ void FlowPart::postResize()
 
 void FlowPart::createStdInput()
 {
-	m_stdInput = (FPNode*)createNode( 0, 0, Node::dir_up, "stdinput", Node::fp_in );
+	m_stdInput = (FPNode*)createNode( 0, 0, 270, "stdinput", Node::fp_in );
 	updateNodePositions();
 }
 void FlowPart::createStdOutput()
 {
-	m_stdOutput = (FPNode*)createNode( 0, 0, Node::dir_down, "stdoutput", Node::fp_out );
+	m_stdOutput = (FPNode*)createNode( 0, 0, 90, "stdoutput", Node::fp_out );
 	updateNodePositions();
 }
 void FlowPart::createAltOutput()
 {
-	m_altOutput = (FPNode*)createNode( 0, 0, Node::dir_right, "altoutput", Node::fp_out );
+	m_altOutput = (FPNode*)createNode( 0, 0, 0, "altoutput", Node::fp_out );
 	updateNodePositions();
 }
 
@@ -174,16 +173,11 @@ void FlowPart::initSymbol( FlowPart::FlowSymbol symbol, int width )
 	switch(symbol)
 	{
 		case FlowPart::ps_other:
-		{
 			return;
-		}
 		case FlowPart::ps_call:
 		case FlowPart::ps_process:
-		{	
 			setItemPoints( QRect( -width/2, -16, width, 24 ) );
 			break;
-		}
-		
 		case FlowPart::ps_io:
 		{
 			// define parallelogram shape
@@ -195,7 +189,6 @@ void FlowPart::initSymbol( FlowPart::FlowSymbol symbol, int width )
 			setItemPoints(pa);
 			break;
 		}
-		
 		case FlowPart::ps_round:
 		{
 			// define rounded rectangles as two semicricles with RP_NUM/2 points with gap inbetween
@@ -248,73 +241,86 @@ void FlowPart::drawShape( QPainter &p )
 	const double w = width();
 	double h = height();
 	
-	switch (m_flowSymbol)
-	{
-		case FlowPart::ps_other:
-		{	
-			CNItem::drawShape(p);
-			break;
-		}
+	switch (m_flowSymbol) {
+
+	case FlowPart::ps_other:
+		CNItem::drawShape(p);
+		break;
+	case FlowPart::ps_io:
+		{
+		h--;
+		double roundSize = 8;
+		double slantIndent = 5;
+			
+// 		CNItem::drawShape(p);
+		double inner = std::atan(h/slantIndent);
+		double outer = M_PI - inner;
 		
-		case FlowPart::ps_io:
-		{
-			h--;
-			double roundSize = 8;
-			double slantIndent = 5;
+		int inner16 = int(16*inner*DPR);
+		int outer16 = int(16*outer*DPR);
 			
-			const double pi = 3.1415926536;
-			const double DPR = 180./pi;
-// 			CNItem::drawShape(p);
-			double inner = std::atan(h/slantIndent);
-			double outer = pi-inner;
-			
-			int inner16 = int(16*inner*DPR);
-			int outer16 = int(16*outer*DPR);
-			
-			p.save();
-			p.setPen( Qt::NoPen );
-			p.drawPolygon( areaPoints() );
-			p.restore();
-			
-			p.drawLine( int(_x+slantIndent+roundSize/2),	int(_y),	int(_x+w-roundSize/2),	int(_y) );
-			p.drawLine( int(_x-slantIndent+w-roundSize/2),	int(_y+h),	int(_x+roundSize/2),	int(_y+h) );
-			p.drawLine( int(_x+w+(std::sin(outer)-1)*(roundSize/2)),				int(_y+(1-std::cos(outer))*(roundSize/2)),
-						int(_x+w-slantIndent+(std::sin(inner)-1)*(roundSize/2)),	int(_y+h+(std::cos(inner)-1)*(roundSize/2)) );
-			p.drawLine( int(_x+(1-std::sin(outer))*(roundSize/2)),					int(_y+h+(std::cos(outer)-1)*(roundSize/2)),
-						int(_x+slantIndent+(1-std::sin(inner))*(roundSize/2)),		int(_y+(1-std::cos(inner))*(roundSize/2)) );
-			
-			p.drawArc( int(_x+slantIndent),				int(_y),				int(roundSize), int(roundSize), 90*16,			inner16 );
-			p.drawArc( int(_x+w-roundSize),				int(_y),				int(roundSize), int(roundSize), 270*16+inner16,	outer16 );
-			p.drawArc( int(_x-slantIndent+w-roundSize),	int(_y+h-roundSize),	int(roundSize), int(roundSize), 270*16,			inner16 );
-			p.drawArc( int(_x),							int(_y+h-roundSize),	int(roundSize), int(roundSize), 90*16+inner16,	outer16) ;
-			break;
-		}
+		p.save();
+		p.setPen( Qt::NoPen );
+		p.drawPolygon( areaPoints() );
+		p.restore();
 		
-		case FlowPart::ps_decision:
-		{
-			// TODO Make the shape nice and pretty with rounded corners
-			CNItem::drawShape(p);
-			break;
-		}
-		
-		case FlowPart::ps_call:
-		{
-			p.drawRoundRect( int(_x), int(_y), int(w), int(h+1), int(1000./w), int(1000./h) );
-			p.drawLine( int(_x+8), int(_y), int(_x+8), int(_y+h) );
-			p.drawLine( int(_x+w-8), int(_y), int(_x+w-8), int(_y+h) );
-			break;
-		}
-		case FlowPart::ps_process:
-		{
-			p.drawRoundRect( int(_x), int(_y), int(w), int(h+1), int(1000./w), int(1000./h) );
-			break;
-		}
-		
-		case FlowPart::ps_round:
-		{
-			p.drawRoundRect( int(_x), int(_y), int(w), int(h+1), 30, 100 );
-			break;
+		p.drawLine(int(_x + slantIndent+roundSize / 2),
+			   int(_y),
+			   int(_x + w  -                                  roundSize / 2),
+			   int(_y));
+		p.drawLine(int(_x + w  - slantIndent  - 		  roundSize / 2),
+			   int(_y + h),
+			   int(_x + 					  roundSize / 2),
+			   int(_y + h) );
+		p.drawLine(int(_x + w  + (std::sin(outer) - 1)          * roundSize / 2),
+			   int(_y + (1 -  std::cos(outer))              * roundSize / 2),
+			   int(_x + w  - slantIndent + (std::sin(inner) - 1) * roundSize/2),
+			   int(_y + h  + (std::cos(inner) - 1)          * roundSize / 2) );
+		p.drawLine(int(_x + (1 -  std::sin(outer))              * roundSize / 2),
+			   int(_y + h  + (std::cos(outer) - 1)          * roundSize / 2),
+			   int(_x + slantIndent + (1 - std::sin(inner)) * roundSize / 2),
+			   int(_y +               (1 - std::cos(inner)) * roundSize / 2) );
+		p.drawArc(int(_x + slantIndent),
+			  int(_y),
+			  int(roundSize),
+			  int(roundSize),
+			  90 * 16,
+			  inner16);
+		p.drawArc(int(_x + w - roundSize),
+			  int(_y),
+			  int(roundSize),
+			  int(roundSize),
+			  270 * 16 + inner16,
+			  outer16 );
+		p.drawArc(int(_x - slantIndent + w - roundSize),
+			  int(_y + h - roundSize),
+			  int(roundSize),
+			  int(roundSize),
+			  270*16,
+			  inner16);
+		p.drawArc(int(_x),
+			  int(_y + h - roundSize),
+			  int(roundSize),
+			  int(roundSize),
+			  90*16 + inner16,
+			  outer16);
+		break;
 		}
+	case FlowPart::ps_decision:
+		// TODO Make the shape nice and pretty with rounded corners
+		CNItem::drawShape(p);
+		break;
+	case FlowPart::ps_call:
+		p.drawRoundRect( int(_x), int(_y), int(w), int(h + 1), int(1000. / w), int(1000. / h) );
+		p.drawLine( int(_x + 8), int(_y), int(_x + 8), int(_y + h) );
+		p.drawLine( int(_x + w - 8), int(_y), int(_x + w - 8), int(_y + h) );
+		break;
+	case FlowPart::ps_process:
+		p.drawRoundRect( int(_x), int(_y), int(w), int(h + 1), int(1000. / w), int(1000. / h) );
+		break;
+	case FlowPart::ps_round:
+		p.drawRoundRect( int(_x), int(_y), int(w), int(h + 1), 30, 100 );
+		break;
 	}
 	
 	p.setPen( Qt::black );
@@ -334,7 +340,9 @@ FlowPart* FlowPart::outputPart( const QString& internalNodeId )
 	Node *node = p_icnDocument->nodeWithID( nodeId(internalNodeId) );
 	
 	FPNode *fpnode = dynamic_cast<FPNode*>(node);
-	if ( !fpnode || fpnode->type() == Node::fp_in )
+		// FIXME dynamic_cast used to replace fpnode::type() call
+	if ( !fpnode || ( dynamic_cast<InputFlowNode*>(fpnode) != 0) )
+	// if ( !fpnode || fpnode->type() == Node::fp_in )
 		return 0l;
 	
 	return fpnode->outputFlowPart();
@@ -354,8 +362,8 @@ FlowPartList FlowPart::inputParts()
 {
 	FlowPartList list;
 	
-	const NodeMap::iterator nEnd = m_nodeMap.end();
-	for ( NodeMap::iterator it = m_nodeMap.begin(); it != nEnd; ++it )
+	const NodeInfoMap::iterator nEnd = m_nodeMap.end();
+	for ( NodeInfoMap::iterator it = m_nodeMap.begin(); it != nEnd; ++it )
 	{
 		Node *node = p_icnDocument->nodeWithID( it.data().id );
 		FlowPartList newList;
@@ -377,8 +385,8 @@ FlowPartList FlowPart::outputParts()
 {
 	FlowPartList list;
 	
-	const NodeMap::iterator end = m_nodeMap.end();
-	for ( NodeMap::iterator it = m_nodeMap.begin(); it != end; ++it )
+	const NodeInfoMap::iterator end = m_nodeMap.end();
+	for ( NodeInfoMap::iterator it = m_nodeMap.begin(); it != end; ++it )
 	{
 		FlowPart *part = outputPart( it.key() );
 		if (part) list.append(part);
@@ -392,8 +400,8 @@ FlowPart* FlowPart::endPart( QStringList ids, FlowPartList *previousParts )
 {
 	if ( ids.empty() )
 	{
-		const NodeMap::iterator end = m_nodeMap.end();
-		for ( NodeMap::iterator it = m_nodeMap.begin(); it != end; ++it )
+		const NodeInfoMap::iterator end = m_nodeMap.end();
+		for ( NodeInfoMap::iterator it = m_nodeMap.begin(); it != end; ++it )
 		{
 			ids.append( it.key() );
 		}
@@ -406,11 +414,13 @@ FlowPart* FlowPart::endPart( QStringList ids, FlowPartList *previousParts )
 	} else if ( previousParts->contains(this) ) {
 		return 0l;
 	}
+
 	previousParts->append(this);
 	
 	if ( ids.empty() ) {
 		return 0l;
 	}
+
 	if ( ids.size() == 1 ) {
 		return outputPart( *(ids.begin()) );
 	}
@@ -499,9 +509,7 @@ void FlowPart::handleIfElse( FlowCode *code, const QString &case1Statement, cons
 			code->addCodeBranch(part2);
 			code->addCode("}");
 		}
-	}
-	else if ( code->isValidBranch(part2) )
-	{
+	} else if ( code->isValidBranch(part2) ) {
 		// Use the case2 statement
 		code->addCode( "if "+case2Statement+" then "+"\n{" );
 		code->addCodeBranch(part2);
@@ -527,8 +535,11 @@ Variant * FlowPart::createProperty( const QString & id, Variant::Type::Value typ
 	
 	if ( type == Variant::Type::VarName )
 	{
-		if ( MicroSettings * settings = m_pFlowCodeDocument->microSettings() )
-			v->setAllowed( settings->variableNames() );
+		if ( m_pFlowCodeDocument )
+		{
+			if ( MicroSettings * settings = m_pFlowCodeDocument->microSettings() )
+				v->setAllowed( settings->variableNames() );
+		}
 		connect( property(id), SIGNAL(valueChanged(QVariant, QVariant )), this, SLOT(varNameChanged(QVariant, QVariant )) );
 	}
 	else
@@ -540,32 +551,31 @@ Variant * FlowPart::createProperty( const QString & id, Variant::Type::Value typ
 
 void FlowPart::slotUpdateFlowPartVariables()
 {
-	if (!m_pFlowCodeDocument)
-		return;
+	if (!m_pFlowCodeDocument) return;
 	
 	MicroSettings *s = m_pFlowCodeDocument->microSettings();
-	if (!s)
-		return;
+	if (!s) return;
 	
 	const PinMappingMap pinMappings = s->pinMappings();
 	QStringList sevenSegMaps;
 	QStringList keyPadMaps;
+
 	PinMappingMap::const_iterator pEnd = pinMappings.end();
 	for ( PinMappingMap::const_iterator it = pinMappings.begin(); it != pEnd; ++it )
 	{
 		switch ( it.data().type() )
 		{
-			case PinMapping::SevenSegment:
-				sevenSegMaps << it.key();
-				break;
-				
-			case PinMapping::Keypad_4x3:
-			case PinMapping::Keypad_4x4:
-				keyPadMaps << it.key();
-				break;
-				
-			case PinMapping::Invalid:
-				break;
+		case PinMapping::SevenSegment:
+			sevenSegMaps << it.key();
+			break;
+			
+		case PinMapping::Keypad_4x3:
+		case PinMapping::Keypad_4x4:
+			keyPadMaps << it.key();
+			break;
+			
+		case PinMapping::Invalid:
+			break;
 		}
 	}
 	
@@ -579,24 +589,17 @@ void FlowPart::slotUpdateFlowPartVariables()
 	for ( VariantDataMap::iterator it = m_variantData.begin(); it != vEnd; ++it )
 	{
 		Variant * v = it.data();
-		if ( !v )
-			continue;
+		if ( !v ) continue;
 		
 		if ( v->type() == Variant::Type::Port )
 			v->setAllowed( ports );
-		
 		else if ( v->type() == Variant::Type::Pin )
 			v->setAllowed( pins );
-		
-		else if ( v->type() == Variant::Type::SevenSegment )
-		{
+		else if ( v->type() == Variant::Type::SevenSegment ) {
 			v->setAllowed( sevenSegMaps );
 			if ( !sevenSegMaps.isEmpty() && !sevenSegMaps.contains( v->value().toString() ) )
 				v->setValue( sevenSegMaps.first() );
-		}
-		
-		else if ( v->type() == Variant::Type::KeyPad )
-		{
+		} else if ( v->type() == Variant::Type::KeyPad ) {
 			v->setAllowed( keyPadMaps );
 			if ( !keyPadMaps.isEmpty() && !keyPadMaps.contains( v->value().toString() ) )
 				v->setValue( keyPadMaps.first() );
@@ -607,13 +610,11 @@ void FlowPart::slotUpdateFlowPartVariables()
 
 void FlowPart::updateVarNames()
 {
-	if (!m_pFlowCodeDocument)
-		return;
-	
+	if (!m_pFlowCodeDocument) return;
+
 	MicroSettings *s = m_pFlowCodeDocument->microSettings();
-	if (!s)
-		return;
-	
+	if (!s) return;
+
 	const QStringList names = s->variableNames();
 	const VariantDataMap::iterator end = m_variantData.end();
 	for ( VariantDataMap::iterator it = m_variantData.begin(); it != end; ++it )
@@ -624,7 +625,6 @@ void FlowPart::updateVarNames()
 	}
 }
 
-
 void FlowPart::varNameChanged( QVariant newValue, QVariant oldValue )
 {
 	if (!m_pFlowCodeDocument)
@@ -632,19 +632,17 @@ void FlowPart::varNameChanged( QVariant newValue, QVariant oldValue )
 	m_pFlowCodeDocument->varNameChanged( newValue.asString(), oldValue.asString() );
 }
 
-
-inline int nodeDirToPos( Node::node_dir dir )
+inline int nodeDirToPos( int dir )
 {
-	switch (dir)
-	{
-		case Node::dir_right:
-			return 0;
-		case Node::dir_up:
-			return 1;
-		case Node::dir_left:
-			return 2;
-		case Node::dir_down:
-			return 3;
+	switch ( dir ) {
+	case 0:
+		return 0;
+	case 270:
+		return 1;
+	case 180:
+		return 2;
+	case 90:
+		return 3;
 	}
 	return 0;
 }
@@ -652,8 +650,7 @@ inline int nodeDirToPos( Node::node_dir dir )
 
 void FlowPart::updateAttachedPositioning( )
 {
-	if (b_deleted)
-		return;
+	if (b_deleted) return;
 	
 	//BEGIN Rearrange text if appropriate
 	const QRect textPos[4] = {
@@ -669,10 +666,10 @@ void FlowPart::updateAttachedPositioning( )
 	Text *outputFalseText = m_textMap.contains("output_false") ? m_textMap["output_false"] : 0l;
 	
 	if ( stdOutputInfo && outputTrueText )
-		outputTrueText->setOriginalRect( textPos[ nodeDirToPos( (Node::node_dir)stdOutputInfo->orientation ) ] );
+		outputTrueText->setOriginalRect( textPos[ nodeDirToPos( stdOutputInfo->orientation ) ] );
 	
 	if ( altOutputInfo && outputFalseText )
-		outputFalseText->setOriginalRect( textPos[ nodeDirToPos( (Node::node_dir)altOutputInfo->orientation ) ] );
+		outputFalseText->setOriginalRect( textPos[ nodeDirToPos( altOutputInfo->orientation ) ] );
 	
 	const TextMap::iterator textMapEnd = m_textMap.end();
 	for ( TextMap::iterator it = m_textMap.begin(); it != textMapEnd; ++it )
@@ -683,8 +680,8 @@ void FlowPart::updateAttachedPositioning( )
 	}
 	//END Rearrange text if appropriate
 	
-	const NodeMap::iterator end = m_nodeMap.end();
-	for ( NodeMap::iterator it = m_nodeMap.begin(); it != end; ++it )
+	const NodeInfoMap::iterator end = m_nodeMap.end();
+	for ( NodeInfoMap::iterator it = m_nodeMap.begin(); it != end; ++it )
 	{
 		if ( !it.data().node )
 		{
@@ -701,11 +698,10 @@ void FlowPart::updateAttachedPositioning( )
 #undef round_8
 			
 		it.data().node->move( int(nx+x()), int(ny+y()) );
-		it.data().node->setOrientation( (Node::node_dir)it.data().orientation );
+		it.data().node->setOrientation( it.data().orientation );
 	}
 }
 
-
 ItemData FlowPart::itemData( ) const
 {
 	ItemData itemData = CNItem::itemData();
@@ -713,7 +709,6 @@ ItemData FlowPart::itemData( ) const
 	return itemData;
 }
 
-
 void FlowPart::restoreFromItemData( const ItemData & itemData )
 {
 	CNItem::restoreFromItemData(itemData);
@@ -721,7 +716,6 @@ void FlowPart::restoreFromItemData( const ItemData & itemData )
 		setOrientation( uint(itemData.orientation) );
 }
 
-
 void FlowPart::updateNodePositions()
 {
 	if ( m_orientation > 7 )
@@ -734,56 +728,46 @@ void FlowPart::updateNodePositions()
 	NodeInfo * stdOutputInfo = m_stdOutput ? &m_nodeMap["stdoutput"] : 0;
 	NodeInfo * altOutputInfo = m_altOutput ? &m_nodeMap["altoutput"] : 0l;
 	
-	if ( m_stdInput && m_stdOutput && m_altOutput )
-	{
+	if ( m_stdInput && m_stdOutput && m_altOutput ) {
 		stdInputInfo->orientation = diamondNodePositioning[m_orientation][0];
 		stdOutputInfo->orientation = diamondNodePositioning[m_orientation][1];
 		altOutputInfo->orientation = diamondNodePositioning[m_orientation][2];
-	}
-	else if ( m_stdInput && m_stdOutput )
-	{
+	} else if ( m_stdInput && m_stdOutput ) {
 		stdInputInfo->orientation = inOutNodePositioning[m_orientation][0];
 		stdOutputInfo->orientation = inOutNodePositioning[m_orientation][1];
-	}
-	else if ( m_orientation < 4 )
-	{
+	} else if ( m_orientation < 4 ) {
 		if (stdInputInfo)
 			stdInputInfo->orientation = inNodePositioning[m_orientation];
 		else if (stdOutputInfo)
 			stdOutputInfo->orientation = outNodePositioning[m_orientation];
-	}
-	else
-	{
+	} else {
 		kdWarning() << k_funcinfo << "Invalid orientation: "<<m_orientation<<endl;
 		return;
 	}
 	
-	const NodeMap::iterator end = m_nodeMap.end();
-	for ( NodeMap::iterator it = m_nodeMap.begin(); it != end; ++it )
+	const NodeInfoMap::iterator end = m_nodeMap.end();
+	for ( NodeInfoMap::iterator it = m_nodeMap.begin(); it != end; ++it )
 	{
 		if ( !it.data().node )
 			kdError() << k_funcinfo << "Node in nodemap is null" << endl;
-		
-		else
-		{
-			switch ( it.data().orientation )
-			{
-				case Node::dir_right:
-					it.data().x = offsetX()+width()+8;
-					it.data().y = 0;
-					break;
-				case Node::dir_up:
-					it.data().x = 0;
-					it.data().y = offsetY()-8;
-					break;
-				case Node::dir_left:
-					it.data().x = offsetX()-8;
-					it.data().y = 0;
-					break;
-				case Node::dir_down:
-					it.data().x = 0;
-					it.data().y = offsetY()+height()+8;;
-					break;
+		else {
+			switch ( it.data().orientation ) {
+			case 0:
+				it.data().x = offsetX()+width()+8;
+				it.data().y = 0;
+				break;
+			case 270:
+				it.data().x = 0;
+				it.data().y = offsetY()-8;
+				break;
+			case 180:
+				it.data().x = offsetX()-8;
+				it.data().y = 0;
+				break;
+			case 90:
+				it.data().x = 0;
+				it.data().y = offsetY()+height()+8;;
+				break;
 			}
 		}
 	}
@@ -899,10 +883,7 @@ void FlowPart::orientationPixmap( uint orientation, QPixmap & pm ) const
 		p.drawLineSegments(cross);
 		maskPainter.drawLineSegments(cross);
 		//END Draw "false" output as a cross
-	}
-	
-	else if ( m_stdInput || m_stdOutput )
-	{
+	} else if ( m_stdInput || m_stdOutput ) {
 		p.drawRoundRect( int(0.3*size.width()), int(0.4*size.height()), int(0.4*size.width()), int(0.2*size.height()) );
 		maskPainter.drawRoundRect( int(0.3*size.width()), int(0.4*size.height()), int(0.4*size.width()), int(0.2*size.height()) );
 		
@@ -929,22 +910,16 @@ void FlowPart::orientationPixmap( uint orientation, QPixmap & pm ) const
 		int inPos = -1;
 		int outPos = -1;
 		
-		if ( m_stdInput && m_stdOutput )
-		{
+		if ( m_stdInput && m_stdOutput ) {
 			inPos = nodeDirToPos( inOutNodePositioning[orientation][0] );
 			outPos = nodeDirToPos( inOutNodePositioning[orientation][1] );
-		}
-		else if ( m_stdInput )
-		{
+		} else if ( m_stdInput ) {
 			inPos = nodeDirToPos( inNodePositioning[orientation] );
-		}
-		else if ( m_stdOutput )
-		{
+		} else if ( m_stdOutput ) {
 			outPos = nodeDirToPos( outNodePositioning[orientation] );
 		}
 		
-		if ( inPos != -1 )
-		{
+		if ( inPos != -1 ) {
 			QPointArray inArrow(6);
 			for ( int i=0; i<6; ++i )
 			{
@@ -955,11 +930,9 @@ void FlowPart::orientationPixmap( uint orientation, QPixmap & pm ) const
 			maskPainter.drawPolygon(inArrow);
 		}
 		
-		if ( outPos != -1 )
-		{
+		if ( outPos != -1 ) {
 			QPointArray outArrow(6);
-			for ( int i=0; i<6; ++i )
-			{
+			for ( int i=0; i<6; ++i ) {
 				outArrow[i] = arrows[outPos][i];
 			}
 			outArrow.translate( d[outPos].x(), d[outPos].y() );
@@ -971,7 +944,6 @@ void FlowPart::orientationPixmap( uint orientation, QPixmap & pm ) const
 	pm.setMask(mask);
 }
 
-
 #include "flowpart.moc"
 
 
