@@ -8,7 +8,9 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 
+#include "canvasitemparts.h"
 #include "canvasmanipulator.h"
+#include "cells.h"
 #include "circuitdocument.h"
 #include "connector.h"
 #include "cnitem.h"
@@ -22,6 +24,7 @@
 #include "ktechlab.h"
 #include "core/ktlconfig.h"
 #include "pin.h"
+#include "resizeoverlay.h"
 #include "simulator.h"
 
 #include <kapplication.h>
@@ -32,6 +35,8 @@
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
 #include <kprinter.h>
+
+#include <qapplication.h>
 #include <qcheckbox.h>
 #include <qclipboard.h>
 #include <qcursor.h>
@@ -44,41 +49,39 @@
 #include <qtimer.h>
 
 #include <cmath>
+#include <cassert>
 
 
 //BEGIN class ItemDocument
 int ItemDocument::m_nextActionTicket = 0;
 
-ItemDocument::ItemDocument( const QString &caption, KTechlab *ktechlab, const char *name)
-	: Document( caption, ktechlab, name )
+ItemDocument::ItemDocument( const QString &caption, const char *name)
+	: Document( caption, name )
 {
 	m_queuedEvents = 0;
-	p_ktechlab = ktechlab;
 	m_nextIdNum = 1;
-	m_savedState = 0l;
-	m_currentState = 0l;
+	m_savedState = 0;
+	m_currentState = 0;
 	m_bIsLoading = false;
 	
 	m_canvas = new Canvas( this, "canvas" );
 	m_canvasTip = new CanvasTip(this,m_canvas);
 	m_cmManager = new CMManager(this);
-	m_undoStack.setAutoDelete(true);
-	m_redoStack.setAutoDelete(true);
 	
 	updateBackground();
-	m_canvas->resize( 0, 0 );
-	m_canvas->setDoubleBuffering(true);
+	
+	m_pUpdateItemViewScrollbarsTimer = new QTimer(this);
+	connect( m_pUpdateItemViewScrollbarsTimer, SIGNAL(timeout()), this, SLOT(updateItemViewScrollbars()) );
 	
 	m_pEventTimer = new QTimer(this);
 	connect( m_pEventTimer, SIGNAL(timeout()), this, SLOT(processItemDocumentEvents()) );
 	
-	connect( this, SIGNAL(itemSelected(Item*)), this, SLOT(slotInitItemActions(Item*)) );
-	connect( this, SIGNAL(itemUnselected(Item*)), this, SLOT(slotInitItemActions(Item*)) );
+	connect( this, SIGNAL(selectionChanged()), this, SLOT(slotInitItemActions()) );
 	
-	connect( ComponentSelector::self(),	SIGNAL(itemClicked(const QString& )),		this, SLOT(slotUnsetRepeatedItemId()) );
-	connect( FlowPartSelector::self(),	SIGNAL(itemClicked(const QString& )),		this, SLOT(slotUnsetRepeatedItemId()) );
+	connect( ComponentSelector::self(),	SIGNAL(itemClicked(const QString& )),	this, SLOT(slotUnsetRepeatedItemId()) );
+	connect( FlowPartSelector::self(),	SIGNAL(itemClicked(const QString& )),	this, SLOT(slotUnsetRepeatedItemId()) );
 #ifdef MECHANICS
-	connect( MechanicsSelector::self(),	SIGNAL(itemClicked(const QString& )),		this, SLOT(slotUnsetRepeatedItemId()) );
+	connect( MechanicsSelector::self(),	SIGNAL(itemClicked(const QString& )),	this, SLOT(slotUnsetRepeatedItemId()) );
 #endif
 
 	m_pAlignmentAction = new KActionMenu( i18n("Alignment"), "rightjust", this );
@@ -86,24 +89,24 @@ ItemDocument::ItemDocument( const QString &caption, KTechlab *ktechlab, const ch
 	slotUpdateConfiguration();
 }
 
-
 ItemDocument::~ItemDocument()
 {
 	m_bDeleted = true;
 	
-	ItemList toDelete = m_itemList;
-	const ItemList::iterator end = toDelete.end();
-	for ( ItemList::iterator it = toDelete.begin(); it != end; ++it )
-		delete *it;
-	
-	delete m_cmManager;
-	m_cmManager = 0l;
-	delete m_currentState;
-	m_currentState = 0l;
-	delete m_canvasTip;
-	m_canvasTip = 0l;
-}
+//	ItemMap toDelete = m_itemList;
 
+	const ItemMap::iterator end = m_itemList.end();
+	for ( ItemMap::iterator it = m_itemList.begin(); it != end; ++it )
+		delete *it;
+	m_itemList.clear();
+
+	cleanClearStack( m_undoStack );
+	cleanClearStack( m_redoStack );
+
+	delete m_cmManager;
+	delete m_currentState;
+	delete m_canvasTip;
+}
 
 void ItemDocument::handleNewView( View * view )
 {
@@ -111,30 +114,21 @@ void ItemDocument::handleNewView( View * view )
 	requestEvent( ItemDocument::ItemDocumentEvent::ResizeCanvasToItems );
 }
 
-
-bool ItemDocument::registerItem( QCanvasItem *qcanvasItem )
+bool ItemDocument::registerItem(QCanvasItem *qcanvasItem)
 {
-	if (!qcanvasItem)
-		return false;
+	if (!qcanvasItem) return false;
 	
 	requestEvent( ItemDocument::ItemDocumentEvent::ResizeCanvasToItems );
 	
-	switch (qcanvasItem->rtti())
+	if(Item *item = dynamic_cast<Item*>(qcanvasItem) )
 	{
-		case ItemDocument::RTTI::DrawPart:
-		case ItemDocument::RTTI::CNItem:
-		{
-			Item *item = dynamic_cast<Item*>(qcanvasItem);
-			m_itemList.append(item);
-			connect( item, SIGNAL(removed(Item*)), this, SLOT(requestRerouteInvalidatedConnectors()) );
-			connect( item, SIGNAL(selected(Item*,bool)), this, SIGNAL(itemSelected(Item*)) );
-			connect( item, SIGNAL(unselected(Item*,bool)), this, SIGNAL(itemUnselected(Item*)) );
-			itemAdded(item);
-			return true;
-		}
-		default:
-			return false;
+		m_itemList[ item->id() ] = item;
+		connect( item, SIGNAL(selectionChanged()), this, SIGNAL(selectionChanged()) );
+		itemAdded(item);
+		return true;
 	}
+	
+	return false;
 }
 
 
@@ -143,20 +137,17 @@ void ItemDocument::slotSetDrawAction( int drawAction )
 	m_cmManager->setDrawAction(drawAction);
 }
 
-
 void ItemDocument::cancelCurrentOperation()
 {
 	m_cmManager->cancelCurrentManipulation();
 }
 
-
 void ItemDocument::slotSetRepeatedItemId( const QString &id )
 {
 	m_cmManager->setCMState( CMManager::cms_repeated_add, true );
 	m_cmManager->setRepeatedAddId(id);
 }
 
-
 void ItemDocument::slotUnsetRepeatedItemId()
 {
 	m_cmManager->setCMState( CMManager::cms_repeated_add, false );
@@ -229,8 +220,8 @@ bool ItemDocument::openURL( const KURL &url )
 	
 	// Load Z-position info
 	m_zOrder.clear();
-	ItemList::iterator end = m_itemList.end();
-	for ( ItemList::iterator it = m_itemList.begin(); it != end; ++it )
+	ItemMap::iterator end = m_itemList.end();
+	for ( ItemMap::iterator it = m_itemList.begin(); it != end; ++it )
 	{
 		if ( !*it || (*it)->parentItem() )
 			continue;
@@ -246,7 +237,7 @@ void ItemDocument::print()
 {
 	static KPrinter * printer = new KPrinter;
 	
-	if ( ! printer->setup( p_ktechlab ) )
+	if ( ! printer->setup( KTechlab::self() ) )
 		return;
 	
 	// setup the printer.  with Qt, you always "print" to a
@@ -258,10 +249,13 @@ void ItemDocument::print()
 	// we let our view do the actual printing
 	QPaintDeviceMetrics metrics( printer );
 	
-	// Round to 16 (= 2 * 8) so that we cut in the middle of squares
-	int w = 16*int(metrics.width()/16);
-	int h = 16*int(metrics.height()/16);
-	
+	// Round to 16 so that we cut in the middle of squares
+	int w = metrics.width();
+	w = (w & 0xFFFFFFF0) + ((w << 1) & 0x10);
+
+	int h = metrics.height();
+	h = (h & 0xFFFFFFF0) + ((h << 1) & 0x10);
+
 	p.setClipping( true );
 	p.setClipRect( 0, 0, w, h, QPainter::CoordPainter );
 	
@@ -295,22 +289,27 @@ void ItemDocument::print()
 	p.end();
 }
 
-
 void ItemDocument::requestStateSave( int actionTicket )
 {
-	if ( m_bIsLoading )
-		return;
-	
-	m_redoStack.clear();
+	if ( m_bIsLoading ) return;
 	
+	cleanClearStack( m_redoStack );
+
 	if ( (actionTicket >= 0) && (actionTicket == m_currentActionTicket) )
 	{
 		delete m_currentState;
-		m_currentState = 0l;
+		m_currentState = 0;
 	}
 	
 	m_currentActionTicket = actionTicket;
 	
+	//FIXME: it is possible, that we push something here, also nothing has changed, yet.
+	// to reproduce do:
+	// 1. select an item -> something is pushed onto undoStack, but nothing changed
+	// 2. select Undo -> pushed on redoStack, pop from undoStack
+	// 3. deselect item -> there is still something on the redoStack
+	//
+	// this way you can fill up the redoStack, as you like :-/
 	if (m_currentState)
 		m_undoStack.push(m_currentState);
 	
@@ -325,28 +324,38 @@ void ItemDocument::requestStateSave( int actionTicket )
 	emit undoRedoStateChanged();
 	
 	//FIXME To resize undo queue, have to pop and push everything
+	// In Qt4 QStack is used and QStack inherits QVector, that should
+	// make it a bit more easy
 	int maxUndo = KTLConfig::maxUndo();
 	if ( maxUndo <= 0 || m_undoStack.count() < (unsigned)maxUndo )
 		return;
 	IDDStack tempStack;
 	int pushed = 0;
-	while ( !m_undoStack.isEmpty() && pushed < maxUndo )
-	{
+	while ( !m_undoStack.isEmpty() && pushed < maxUndo ) {
 		tempStack.push( m_undoStack.pop() );
 		pushed++;
 	}
-	m_undoStack.clear();
+	cleanClearStack( m_undoStack );
 	while ( !tempStack.isEmpty() )
 		m_undoStack.push( tempStack.pop() );
 }
 
+void ItemDocument::cleanClearStack( IDDStack &stack )
+{
+	while ( !stack.isEmpty() )
+	{
+		ItemDocumentData * idd = stack.pop();
+		if ( m_currentState != idd )
+			delete idd;
+	}
+}
 
 void ItemDocument::clearHistory()
 {
-	m_undoStack.clear();
-	m_redoStack.clear();
+	cleanClearStack( m_undoStack );
+	cleanClearStack( m_redoStack );
 	delete m_currentState;
-	m_currentState = 0l;
+	m_currentState = 0;
 	requestStateSave();
 }
 
@@ -366,25 +375,21 @@ bool ItemDocument::isRedoAvailable() const
 void ItemDocument::undo()
 {
 	ItemDocumentData *idd = m_undoStack.pop();
-	if (!idd)
-		return;
-	
-	if (m_currentState)
-		m_redoStack.push(m_currentState);
-	
+	if (!idd) return;
+
+	if (m_currentState) m_redoStack.push(m_currentState);
+
 	idd->restoreDocument(this);
 	m_currentState = idd;
-	
+
 	setModified( m_savedState != m_currentState );
 	emit undoRedoStateChanged();
 }
 
-
 void ItemDocument::redo()
 {
 	ItemDocumentData *idd = m_redoStack.pop();
-	if (!idd)
-		return;
+	if (!idd) return;
 	
 	if (m_currentState)
 		m_undoStack.push(m_currentState);
@@ -396,7 +401,6 @@ void ItemDocument::redo()
 	emit undoRedoStateChanged();
 }
 
-
 void ItemDocument::cut()
 {
     copy();
@@ -413,27 +417,26 @@ void ItemDocument::paste()
 	unselectAll();
 	
 	ItemDocumentData data( type() );
-	data.fromXML(xml);
+	
+	if ( !data.fromXML(xml) )
+		return;
+	
 	data.generateUniqueIDs(this);
-	data.translateContents( 64, 64 );
+//	data.translateContents( 64, 64 );
 	data.mergeWithDocument( this, true );
 	
 	// Get rid of any garbage that shouldn't be around / merge connectors / etc
 	flushDeleteList();
-	
+
 	requestStateSave();
 }
 
 
 Item *ItemDocument::itemWithID( const QString &id )
 {
-	const ItemList::iterator end = m_itemList.end();
-	for ( ItemList::iterator it = m_itemList.begin(); it != end; ++it )
-	{
-		if ( (*it)->id() == id )
-			return *it;
-	}
-	return 0L;
+	if ( m_itemList.contains( id ) )
+		return m_itemList[id];
+	else	return 0;
 }
 
 
@@ -445,8 +448,8 @@ void ItemDocument::unselectAll()
 
 void ItemDocument::select( QCanvasItem * item )
 {
-	if (!item)
-		return;
+	if (!item) return;
+
 	item->setSelected( selectList()->contains( item ) || selectList()->addQCanvasItem( item ) );
 }
 
@@ -478,74 +481,72 @@ void ItemDocument::slotUpdateConfiguration()
 QCanvasItem* ItemDocument::itemAtTop( const QPoint &pos ) const
 {
 	QCanvasItemList list = m_canvas->collisions( QRect( pos.x()-1, pos.y()-1, 3, 3 ) );
-	
 	QCanvasItemList::const_iterator it = list.begin();
 	const QCanvasItemList::const_iterator end = list.end();
-	while ( it != end )
-	{
+
+	while ( it != end ) {
 		QCanvasItem *item = *it;
-		if ( item == m_canvasTip ||
-				   item->rtti() == QCanvasItem::Rtti_Line ||
-				   item->rtti() == QCanvasItem::Rtti_Text ||
-				   item->rtti() == QCanvasItem::Rtti_Rectangle ) 
+		if(	!dynamic_cast<Item*>(item) &&
+			!dynamic_cast<ConnectorLine*>(item) &&
+			!dynamic_cast<Node*>(item) &&
+			!dynamic_cast<Widget*>(item) &&
+			!dynamic_cast<ResizeHandle*>(item) )
 		{
 			++it;
-		}
-		else
-		{
-			if ( item->rtti() == ItemDocument::RTTI::ConnectorLine )
-				return (static_cast<ConnectorLine*>(item))->parent();
+		} else {
+			if ( ConnectorLine * l = dynamic_cast<ConnectorLine*>(item) )
+				return l->parent();
 			
 			return item;
 		}
 	}
-	
-	return 0L;
+
+	return 0;
 }
 
 
-
+// these look dangerous., see todo in header file.
 void ItemDocument::alignHorizontally( )
 {
 	selectList()->slotAlignHorizontally();
 	if ( ICNDocument *icnd = dynamic_cast<ICNDocument*>(this) )
 		icnd->requestRerouteInvalidatedConnectors();
 }
+
 void ItemDocument::alignVertically( )
 {
 	selectList()->slotAlignVertically();
 	if ( ICNDocument *icnd = dynamic_cast<ICNDocument*>(this) )
 		icnd->requestRerouteInvalidatedConnectors();
 }
+
 void ItemDocument::distributeHorizontally( )
 {
 	selectList()->slotDistributeHorizontally();
 	if ( ICNDocument *icnd = dynamic_cast<ICNDocument*>(this) )
 		icnd->requestRerouteInvalidatedConnectors();
 }
+
 void ItemDocument::distributeVertically( )
 {
 	selectList()->slotDistributeVertically();
 	if ( ICNDocument *icnd = dynamic_cast<ICNDocument*>(this) )
 		icnd->requestRerouteInvalidatedConnectors();
 }
+// ###########################
+
 
 
 bool ItemDocument::registerUID( const QString &UID )
 {
-	if ( m_idList.findIndex(UID) == -1 )
-	{
-		m_idList.append(UID);
-		return true;
-	}
-	
-	return false;
+	return m_idList.insert(UID).second;
 }
 
 
 void ItemDocument::unregisterUID( const QString & uid )
 {
-	m_idList.remove(uid);
+	m_idList.erase(uid);
+	m_itemList.remove(uid);
 }
 
 
@@ -553,35 +554,33 @@ QString ItemDocument::generateUID( QString name )
 {
 	name.remove( QRegExp("__.*") ); // Change 'node__13' to 'node', for example
 	QString idAttempt = name;
-// 	if ( idAttempt.find("__") != -1 ) idAttempt.truncate( idAttempt.find("__") );
-	while ( !registerUID(idAttempt) ) { idAttempt = name + "__" + QString::number(m_nextIdNum++); }
+	
+	while ( !registerUID(idAttempt) )
+		idAttempt = name + "__" + QString::number(m_nextIdNum++);
 	
 	return idAttempt;
 }
 
-
+// FIXME: popup menu doesn't seem to work these days. =( 
 void ItemDocument::canvasRightClick( const QPoint &pos, QCanvasItem* item )
 {
-	if (item)
-	{
-		if ( item->rtti() == ItemDocument::RTTI::CNItem &&
+	if (item) {
+		if ( dynamic_cast<CNItem*>(item) &&
 			!item->isSelected() )
 		{
 			unselectAll();
 			select(item);
 		}
 	}
-	
-	p_ktechlab->unplugActionList("alignment_actionlist");
-	p_ktechlab->unplugActionList("orientation_actionlist");
-	p_ktechlab->unplugActionList("component_actionlist");
+
+	KTechlab::self()->unplugActionList("alignment_actionlist");
+	KTechlab::self()->unplugActionList("orientation_actionlist");
 	fillContextMenu(pos);
-	
-	QPopupMenu *pop = static_cast<QPopupMenu*>(p_ktechlab->factory()->container("item_popup", p_ktechlab));
-	
-	if (!pop)
-		return;
-	
+
+	QPopupMenu *pop = static_cast<QPopupMenu*>(KTechlab::self()->factory()->container("item_popup", KTechlab::self() ));
+
+	if (!pop) return;
+
 	pop->popup(pos);
 }
 
@@ -591,7 +590,7 @@ void ItemDocument::fillContextMenu( const QPoint & pos )
 	Q_UNUSED(pos);
 	
 	ItemView * activeItemView = dynamic_cast<ItemView*>(activeView());
-	if ( !p_ktechlab || !activeItemView )
+	if ( !KTechlab::self() || !activeItemView )
 		return;
 	
 	KAction * align_actions[] = { 
@@ -601,9 +600,8 @@ void ItemDocument::fillContextMenu( const QPoint & pos )
 		activeItemView->action("distribute_vertically") };
 	
 	bool enableAlignment = selectList()->itemCount() > 1;
-	
-	if ( !enableAlignment )
-		return;
+
+	if ( !enableAlignment ) return;
 	
 	for ( unsigned i = 0; i < 4; ++i )
 	{
@@ -613,16 +611,14 @@ void ItemDocument::fillContextMenu( const QPoint & pos )
 	}
 	QPtrList<KAction> alignment_actions;
 	alignment_actions.append( m_pAlignmentAction );
-	p_ktechlab->plugActionList( "alignment_actionlist", alignment_actions );
+	KTechlab::self()->plugActionList( "alignment_actionlist", alignment_actions );
 }
 
 
-void ItemDocument::slotInitItemActions( Item *item )
+void ItemDocument::slotInitItemActions()
 {
-	Q_UNUSED(item);
-	
 	ItemView * activeItemView = dynamic_cast<ItemView*>(activeView());
-	if ( !p_ktechlab || !activeItemView )
+	if ( !KTechlab::self() || !activeItemView )
 		return;
 	
 	KAction * align_actions[] = { 
@@ -676,6 +672,12 @@ void ItemDocument::updateBackground()
 }
 
 
+void ItemDocument::requestCanvasResize()
+{
+	requestEvent( ItemDocumentEvent::ResizeCanvasToItems );
+}
+
+
 void ItemDocument::requestEvent( ItemDocumentEvent::type type )
 {
 	m_queuedEvents |= type;
@@ -708,50 +710,55 @@ void ItemDocument::processItemDocumentEvents()
 
 void ItemDocument::resizeCanvasToItems()
 {
-	const ViewList::iterator end = m_viewList.end();
-	
 	QRect bound = canvasBoundingRect();
-	QSize size( bound.right(), bound.bottom() );
 	
 	m_viewList.remove((View*)0);
-	
-	for ( ViewList::iterator it = m_viewList.begin(); it != end; ++it )
-	{
-		CVBEditor * cvbEditor = (static_cast<ItemView*>((View*)*it))->cvbEditor();
+	const ViewList::iterator end = m_viewList.end();
+	for ( ViewList::iterator it = m_viewList.begin(); it != end; ++it ) {
+		ItemView * iv = static_cast<ItemView*>((View*)*it);
+		CVBEditor * cvbEditor = iv->cvbEditor();
 		
-		int contentsX, contentsY;
-		int contentsWMX, contentsWMY;
+		QPoint topLeft = iv->mousePosToCanvasPos( QPoint( 0, 0 ) );
+		int width = int( cvbEditor->visibleWidth() / iv->zoomLevel() );
+		int height = int( cvbEditor->visibleHeight() / iv->zoomLevel() );
+		QRect r( topLeft, QSize( width, height ) );
 		
- 		cvbEditor->viewportToContents( cvbEditor->viewport()->width(), cvbEditor->viewport()->height(), contentsX, contentsY );
-		cvbEditor->inverseWorldMatrix().map( contentsX, contentsY, &contentsWMX, &contentsWMY );
+		bound |= r;
 		
-		// Hack to fix a bug whereby when scrolled, but emoty gap before scrollbars,
-		// size slowly decreases one pixel at a time
-		if ( (contentsX - contentsWMX) == 1 )
-			contentsWMX = contentsX;
-		if ( (contentsY - contentsWMY) == 1 )
-			contentsWMY = contentsY;
-		
-		size = size.expandedTo( QSize( contentsWMX, contentsWMY ) );
+// 		kdDebug() << "r="<<r<<endl;
+// 		kdDebug() << "bound="<<bound<<endl;
 	}
 	
-	// We want to avoid flicker....
+	// Make it so that the rectangular offset is a multiple of 8
+	bound.setLeft( bound.left() - (bound.left()%8) );
+	bound.setTop( bound.top() - (bound.top()%8) );
+	
+	m_pUpdateItemViewScrollbarsTimer->start( 10, true );
+	
+	bool changedSize = canvas()->rect() != bound;
+	if ( changedSize ) {
+		canvas()->resize( bound );
+		requestEvent( ItemDocumentEvent::ResizeCanvasToItems );
+	} else if ( ICNDocument * icnd = dynamic_cast<ICNDocument*>(this) ) {
+		icnd->createCellMap();
+	}
+}
+
+
+void ItemDocument::updateItemViewScrollbars()
+{
+	int w = canvas()->width();
+	int h = canvas()->height();
+	
+	const ViewList::iterator end = m_viewList.end();
 	for ( ViewList::iterator it = m_viewList.begin(); it != end; ++it )
 	{
 		ItemView * itemView = static_cast<ItemView*>((View*)*it);
 		CVBEditor * cvbEditor = itemView->cvbEditor();
 		
-		cvbEditor->setVScrollBarMode( ((size.height()*itemView->zoomLevel()) > cvbEditor->visibleHeight()) ? QScrollView::AlwaysOn : QScrollView::AlwaysOff );
-		cvbEditor->setHScrollBarMode( ((size.width()*itemView->zoomLevel()) > cvbEditor->visibleWidth()) ? QScrollView::AlwaysOn : QScrollView::AlwaysOff );
+		cvbEditor->setVScrollBarMode( ((h*itemView->zoomLevel()) > cvbEditor->visibleHeight()) ? QScrollView::AlwaysOn : QScrollView::AlwaysOff );
+		cvbEditor->setHScrollBarMode( ((w*itemView->zoomLevel()) > cvbEditor->visibleWidth()) ? QScrollView::AlwaysOn : QScrollView::AlwaysOff );
 	}
-	
-	bool changedSize = canvas()->size() != size;
-	canvas()->resize( size.width(), size.height() );
-	
-	if (changedSize)
-		requestEvent( ItemDocumentEvent::ResizeCanvasToItems );
-	else if ( ICNDocument * icnd = dynamic_cast<ICNDocument*>(this) )
-		icnd->createCellMap();
 }
 
 
@@ -759,12 +766,36 @@ QRect ItemDocument::canvasBoundingRect() const
 {
 	QRect bound;
 	
+	// Don't include items used for dragging
+	Item *dragItem = 0;
+	const ViewList::const_iterator viewsEnd = m_viewList.end();
+	for ( ViewList::const_iterator it = m_viewList.begin(); it != viewsEnd; ++it )
+	{
+		dragItem = (static_cast<ItemView*>((View*)*it))->dragItem();
+		if ( dragItem ) break;
+	}
+	
 	const QCanvasItemList allItems = canvas()->allItems();
 	const QCanvasItemList::const_iterator end = allItems.end();
+
 	for ( QCanvasItemList::const_iterator it = allItems.begin(); it != end; ++it )
 	{
-		if ( !(*it)->isVisible() )
-			continue;
+		if( !(*it)->isVisible() ) continue;
+		
+		if(dragItem ) {
+			if(*it == dragItem ) continue;
+			
+			if(Node *n = dynamic_cast<Node*>(*it) ) {
+				if ( n->parentItem() == dragItem )
+					continue;
+			}
+		
+			if(GuiPart *gp = dynamic_cast<GuiPart*>(*it) ) {
+				if ( gp->parent() == dragItem )
+					continue;
+			}
+		}
+		
 		bound |= (*it)->boundingRect();
 	}
 	
@@ -793,21 +824,17 @@ void ItemDocument::exportToImage()
 	// gotme here, KFileDialog makes itself parent so tries to destroy cropCheck when it is deleted.
 	// therefore we use a pointer.
 	QString cropMessage;
-	if ( type() == Document::dt_flowcode )
-		cropMessage = i18n("Crop image to program parts");
+
+	cropMessage = i18n("Crop image");
 	
-	else if ( type() == Document::dt_circuit )
-		cropMessage = i18n("Crop image to circuit components");
-	
-	else 
-		cropMessage = i18n("Crop image");
-	
-	QCheckBox *cropCheck = new QCheckBox( cropMessage, p_ktechlab, "cropCheck" );
+	QCheckBox *cropCheck = new QCheckBox( cropMessage, KTechlab::self(), "cropCheck" );
 	cropCheck->setChecked(true); // yes by default?
 	
 	// we need an object so we can retrieve which image type was selected by the user
 	// so setup the filedialog.
-	KFileDialog exportDialog(QString::null, "*.png|PNG Image\n*.bmp|BMP Image\n*.svg|SVG Image" , p_ktechlab, i18n("Export As Image"), true, cropCheck);
+	QString f;
+	f = QString("*.png|%1\n*.bmp|%2\n*.svg|%3").arg( i18n("PNG Image") ).arg( i18n("BMP Image") ).arg( i18n("SVG Image") );
+	KFileDialog exportDialog(QString::null, f, KTechlab::self(), i18n("Export As Image"), true, cropCheck);
 	
 	exportDialog.setOperationMode( KFileDialog::Saving );
 	// now actually show it
@@ -815,12 +842,11 @@ void ItemDocument::exportToImage()
 		return;
 	KURL url = exportDialog.selectedURL();
 
-	if ( url.isEmpty() )
-		return;
+	if ( url.isEmpty() ) return;
 	
 	if ( QFile::exists( url.path() ) )
 	{
-		int query = KMessageBox::warningYesNo( p_ktechlab, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ).arg( url.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ), KStdGuiItem::cancel() );
+		int query = KMessageBox::warningYesNo( KTechlab::self(), i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ).arg( url.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ), KStdGuiItem::cancel() );
 		if ( query == KMessageBox::No ) return;
 	}
 	
@@ -829,9 +855,6 @@ void ItemDocument::exportToImage()
 	// or paper
 	
 	// needs to be something like QPicture to do SVG etc...
-	// at the moment the pixmap is just as big as the canvas,
-	// intend to make some kind of cropping thing so it just 
-	// takes the bit with the circuit on.
 	
 	QRect saveArea;
 	QString type;
@@ -841,35 +864,25 @@ void ItemDocument::exportToImage()
 	filter = filter.lower(); // gently soften the appearance of the letters.
 	
 	// did have a switch here but seems you can't use that on strings
-	if ( filter == "*.png" )
-		type = "PNG";
-	
-	else if ( filter == "*.bmp" )
-		type = "BMP";
-	
-	else if ( filter == "*.svg" )
-	{
+	if ( filter == "*.png") 	type = "PNG";
+	else if ( filter == "*.bmp")	type = "BMP";
+	else if ( filter == "*.svg" ) {
 		KMessageBox::information( NULL, i18n("SVG export is sub-functional"), i18n("Export As Image") );
 		type = "SVG";
 	}
 	// I don't like forcing people to use the right extension (personally)
 	// but it is the easiest way to decide image type.
-	else
-	{
+	else {
 		KMessageBox::sorry( NULL, i18n("Unknown extension, please select one from the filter list."), i18n("Export As Image") );
 		return;
 	}
 
-	if ( cropCheck->isChecked() )
-	{
+	if ( cropCheck->isChecked() ) {
 		cropArea = canvasBoundingRect();
-		if ( cropArea.isNull() )
-		{  
+		if ( cropArea.isNull() ) {
 			KMessageBox::sorry( 0l, i18n("There is nothing to crop"), i18n("Export As Image") );
 			return;
-		}
-		else
-		{
+		} else {
 			cropArea &= canvas()->rect();
 		}
 	}
@@ -878,16 +891,12 @@ void ItemDocument::exportToImage()
 
 	if ( type == "PNG" || type == "BMP" )
 		outputImage = new QPixmap( saveArea.size() );
-	
-	else if ( type == "SVG" )
-	{
+	else if ( type == "SVG" ) {
 		setSVGExport(true);
 		outputImage = new QPicture();
 		// svg can't be cropped using the qimage method.
 		saveArea = cropArea;
-	}
-	else
-	{
+	} else {
 		kdWarning() << "Unknown type!" << endl;
 		return;
 	}
@@ -908,31 +917,25 @@ void ItemDocument::exportToImage()
 	{
 		if( type == "SVG" )
 			saveResult = dynamic_cast<QPicture*>(outputImage)->save( url.path(), type);
-		
-		else
-		{
+		else {
 			QImage img = dynamic_cast<QPixmap*>(outputImage)->convertToImage();
 			img = img.copy(cropArea);
 			saveResult = img.save(url.path(),type);
 		}
-	}
-	else
-	{
+	} else {
 		if ( type=="SVG" )
 			saveResult = dynamic_cast<QPicture*>(outputImage)->save( url.path(), type );
-		else
-			saveResult = dynamic_cast<QPixmap*>(outputImage)->save( url.path(), type );
+		else	saveResult = dynamic_cast<QPixmap*>(outputImage)->save( url.path(), type );
 	}
 	
 	//if(saveResult == true)	KMessageBox::information( this, i18n("Sucessfully exported to \"%1\"").arg( url.filename() ), i18n("Image Export") );
 	//else KMessageBox::information( this, i18n("Export failed"), i18n("Image Export") );
 	
-	if ( type == "SVG" )
-		setSVGExport(false);
-	
+	if ( type == "SVG" ) setSVGExport(false);
+
 	if (saveResult == false)
-		KMessageBox::information( p_ktechlab, i18n("Export failed"), i18n("Image Export") );
-	
+		KMessageBox::information( KTechlab::self(), i18n("Export failed"), i18n("Image Export") );
+
 	delete outputImage;
 }
 
@@ -955,17 +958,14 @@ void ItemDocument::raiseZ()
 }
 void ItemDocument::raiseZ( const ItemList & itemList )
 {
-	if ( m_zOrder.isEmpty() )
-		slotUpdateZOrdering();
+	if ( m_zOrder.isEmpty() ) slotUpdateZOrdering();
 	
-	if ( m_zOrder.isEmpty() )
-		return;
+	if ( m_zOrder.isEmpty() ) return;
 	
 	IntItemMap::iterator begin = m_zOrder.begin();
 	IntItemMap::iterator previous = m_zOrder.end();
 	IntItemMap::iterator it = --m_zOrder.end();
-	do
-	{
+	do {
 		Item * previousData = (previous == m_zOrder.end()) ? 0l : previous.data();
 		Item * currentData = it.data();
 		
@@ -977,8 +977,7 @@ void ItemDocument::raiseZ( const ItemList & itemList )
 		
 		previous = it;
 		--it;
-	}
-	while ( previous != begin );
+	} while ( previous != begin );
 	
 	slotUpdateZOrdering();
 }
@@ -988,13 +987,12 @@ void ItemDocument::lowerZ()
 {
 	lowerZ( selectList()->items(true) );
 }
-void ItemDocument::lowerZ( const ItemList & itemList )
+
+void ItemDocument::lowerZ( const ItemList &itemList )
 {
-	if ( m_zOrder.isEmpty() )
-		slotUpdateZOrdering();
+	if ( m_zOrder.isEmpty() ) slotUpdateZOrdering();
 	
-	if ( m_zOrder.isEmpty() )
-		return;
+	if ( m_zOrder.isEmpty() ) return;
 	
 	IntItemMap::iterator previous = m_zOrder.begin();
 	IntItemMap::iterator end = m_zOrder.end();
@@ -1024,8 +1022,7 @@ void ItemDocument::itemAdded( Item * )
 
 void ItemDocument::slotUpdateZOrdering()
 {
-	ItemList toAdd = m_itemList;
-	toAdd.remove((Item*)0l);
+	ItemMap toAdd = m_itemList;
 	
 	IntItemMap newZOrder;
 	int atLevel = 0;
@@ -1034,17 +1031,16 @@ void ItemDocument::slotUpdateZOrdering()
 	for ( IntItemMap::iterator it = m_zOrder.begin(); it != zEnd; ++it )
 	{	
 		Item * item = it.data();
-		if (!item)
-			continue;
+		if (!item) continue;
 		
-		toAdd.remove(item);
+		toAdd.remove( item->id() );
 		
 		if ( !item->parentItem() && item->isMovable() )
 			newZOrder[atLevel++] = item;
 	}
 	
-	ItemList::iterator addEnd = toAdd.end();
-	for ( ItemList::iterator it = toAdd.begin(); it != addEnd; ++it )
+	ItemMap::iterator addEnd = toAdd.end();
+	for ( ItemMap::iterator it = toAdd.begin(); it != addEnd; ++it )
 	{
 		Item * item = *it;
 		if ( item->parentItem() || !item->isMovable() )
@@ -1063,24 +1059,35 @@ void ItemDocument::slotUpdateZOrdering()
 
 void ItemDocument::update( )
 {
-	ItemList::iterator end = m_itemList.end();
-	for ( ItemList::iterator it = m_itemList.begin(); it != end; ++it )
+	ItemMap::iterator end = m_itemList.end();
+	for ( ItemMap::iterator it = m_itemList.begin(); it != end; ++it )
 	{
-		if ( (*it)->hasDynamicContent() )
+		if ( (*it)->contentChanged() )
 			(*it)->setChanged();
 	}
 }
+
+
+ItemList ItemDocument::itemList( ) const
+{
+	ItemList l;
+	
+	ItemMap::const_iterator end = m_itemList.end();
+	for ( ItemMap::const_iterator it = m_itemList.begin(); it != end; ++it )
+		l << it.data();
+
+	return l;
+}
 //END class ItemDocument
 
 
 
 //BEGIN class CanvasTip
 CanvasTip::CanvasTip( ItemDocument *itemDocument, QCanvas *qcanvas )
-	: QCanvasText(qcanvas)
+	: QCanvasRectangle( qcanvas )
 {
 	p_itemDocument = itemDocument;
-	
-	setColor( Qt::black );
+
 	setZ( ICNDocument::Z::Tip );
 }
 
@@ -1149,8 +1156,7 @@ void CanvasTip::display( const QPoint &pos )
 {
 	unsigned num = m_v.size();
 	
-	for ( unsigned i = 0; i < num; i++ )
-	{
+	for ( unsigned i = 0; i < num; i++ ) {
 		if ( !std::isfinite(m_v[i]) || std::abs(m_v[i]) < 1e-9 )
 			m_v[i] = 0.;
 		
@@ -1160,17 +1166,14 @@ void CanvasTip::display( const QPoint &pos )
 	
 	move( pos.x()+20, pos.y()+4 );
 	
-	if ( num == 0 )
-		return;
+	if ( num == 0 ) return;
 	
 	if ( num == 1 )
 		setText( displayText(0) );
-	
-	else
-	{
+	else {
 		QString text;
 		for ( unsigned i = 0; i < num; i++ )
-			text += QString(" %1: %2\n").arg( QString::number(i) ).arg( displayText(i) );
+			text += QString("%1: %2\n").arg( QString::number(i) ).arg( displayText(i) );
 		setText(text);
 	}
 }
@@ -1181,7 +1184,7 @@ QString CanvasTip::displayText( unsigned num ) const
 	if ( m_v.size() <= num )
 		return QString::null;
 	
-	return QString(" %1%2V  %3%4A ")
+	return QString("%1%2V  %3%4A")
 			.arg( QString::number( m_v[num] / CNItem::getMultiplier(m_v[num]), 'g', 3 ) )
 			.arg( CNItem::getNumberMag( m_v[num] ) )
 			.arg( QString::number( m_i[num] / CNItem::getMultiplier(m_i[num]), 'g', 3 ) )
@@ -1198,13 +1201,25 @@ void CanvasTip::draw( QPainter &p )
 	p.setBrush( QColor( 0xff, 0xff, 0xdc ) );
 	p.setPen( Qt::black );
 	p.drawRect( boundingRect() );
-	QCanvasText::draw(p);
+	
+	QRect textRect = boundingRect();
+	textRect.setLeft( textRect.left() + 3 );
+	textRect.setTop( textRect.top() + 1 );
+	p.drawText( textRect, 0, m_text );
+}
+
+
+void CanvasTip::setText( const QString & text )
+{
+	m_text = text;
+	canvas()->setChanged( boundingRect() );
+	
+	QRect r = QFontMetrics( qApp->font() ).boundingRect( 0, 0, 0, 0, 0, m_text );
+	setSize( r.width() + 4, r.height() - 1 );
 }
 //END class CanvasTip
 
 
-
-
 //BEGIN class Canvas
 Canvas::Canvas( ItemDocument *itemDocument, const char * name )
 	: QCanvas( itemDocument, name )
@@ -1215,15 +1230,23 @@ Canvas::Canvas( ItemDocument *itemDocument, const char * name )
 }
 
 
+void Canvas::resize( const QRect & size )
+{
+	if ( rect() == size )
+		return;
+	QRect oldSize = rect();
+	QCanvas::resize( size );
+	emit resized( oldSize, size );
+}
+
+
 void Canvas::setMessage( const QString & message )
 {
 	m_message = message;
 	
 	if ( message.isEmpty() )
 		m_pMessageTimeout->stop();
-	
-	else
-		m_pMessageTimeout->start( 2000, true );
+	else	m_pMessageTimeout->start( 2000, true );
 	
 	setAllChanged();
 }
@@ -1237,17 +1260,26 @@ void Canvas::drawBackground ( QPainter &p, const QRect & clip )
 	const int ecx = (int)((clip.right()+4)/8);
 	const int scy = (int)((clip.top()-4)/8);
 	const int ecy = (int)((clip.bottom()+4)/8);
-	if ( !((ICNDocument*)(p_itemDocument))->isValidCellReference( scx, scy ) ||
-			   !((ICNDocument*)(p_itemDocument))->isValidCellReference( ecx, ecy ) ) return;
-	Cells *c = ((ICNDocument*)(p_itemDocument))->cells();
+	
+	ICNDocument * icnd = dynamic_cast<ICNDocument*>(p_itemDocument);
+	if ( !icnd )
+		return;
+	
+	Cells * c = icnd->cells();
+	
+	if ( !c->haveCell( scx, scy ) || !c->haveCell( ecx, ecy ) )
+		return;
+	
 	for ( int x=scx; x<=ecx; x++ )
 	{
 		for ( int y=scy; y<=ecy; y++ )
 		{
-			const double score = (*c)[x][y].CIpenalty+(*c)[x][y].Cpenalty;
+			const double score = c->cell( x, y ).CIpenalty + c->cell( x, y ).Cpenalty;
 			int value = (int)std::log(score)*20;
-			if ( value>255 ) value=255;
-			else if (value<0 ) value=0;
+			if ( value>255 )
+				value=255;
+			else if (value<0 )
+				value=0;
 			p.setBrush( QColor( 255, (255-value), (255-value) ) );
 			p.setPen( Qt::NoPen );
 			p.drawRect( (x*8), (y*8), 8, 8 );
@@ -1263,9 +1295,7 @@ void Canvas::drawForeground ( QPainter &p, const QRect & clip )
 	
 	if ( !m_pMessageTimeout->isActive() )
 		return;
-	
-	
-	
+
 	// Following code stolen and adapted from amarok/src/playlist.cpp :)
 	
 	// Find out width of smallest view
@@ -1275,27 +1305,23 @@ void Canvas::drawForeground ( QPainter &p, const QRect & clip )
 	View * firstView = 0l;
 	for ( ViewList::const_iterator it = viewList.begin(); it != end; ++it )
 	{
-		if ( !*it )
-			continue;
+		if ( !*it ) continue;
 		
 		if ( !firstView )
 		{
 			firstView = *it;
 			minSize = (*it)->size();
-		}
-		else
-			minSize = minSize.boundedTo( (*it)->size() );
+		} else	minSize = minSize.boundedTo( (*it)->size() );
 	}
 	
-	if ( !firstView )
-		return;
+	if ( !firstView ) return;
 	
 	QSimpleRichText * t = new QSimpleRichText( m_message, QApplication::font() );
 	
 	int w = t->width();
 	int h = t->height();
-	int x = 15;
-	int y = 15;
+	int x = rect().left() + 15;
+	int y = rect().top() + 15;
 	int b = 10; // text padding
 	
 	if ( w+2*b >= minSize.width() || h+2*b >= minSize.height() )
@@ -1320,4 +1346,3 @@ void Canvas::update()
 
 #include "itemdocument.moc"
 
-
