@@ -9,11 +9,13 @@
  ***************************************************************************/
 
 #include "connector.h"
-#include "ecnode.h"
+#include "junctionnode.h"
 #include "ecsubcircuit.h"
+#include "electronicconnector.h"
 #include "flowcodedocument.h"
+#include "flowconnector.h"
 #include "flowcontainer.h"
-#include "fpnode.h"
+#include "junctionflownode.h"
 #include "itemdocumentdata.h"
 #include "itemlibrary.h"
 #include "picitem.h"
@@ -968,10 +970,10 @@ void ItemDocumentData::mergeWithDocument( ItemDocument *itemDocument, bool selec
 			{
 				QString id = it.key();
 				if ( itemDocument->type() == Document::dt_circuit )
-					new ECNode( icnd, Node::ec_junction, Node::dir_up, QPoint( int(it.data().x), int(it.data().y) ), &id );
+					new JunctionNode( icnd, 270, QPoint( int(it.data().x), int(it.data().y) ), &id );
 			
 				else if ( itemDocument->type() == Document::dt_flowcode )
-					new FPNode( icnd, Node::fp_junction, Node::dir_up, QPoint( int(it.data().x), int(it.data().y) ), &id );
+					new JunctionFlowNode( icnd, 270, QPoint( int(it.data().x), int(it.data().y) ), &id );
 			}
 		}
 		for ( NodeDataMap::iterator it = m_nodeDataMap.begin(); it != nodeEnd; ++it )
@@ -1061,10 +1063,34 @@ void ItemDocumentData::mergeWithDocument( ItemDocument *itemDocument, bool selec
 			}
 			else
 			{
-				Connector *connector = new Connector( startNode, endNode, icnd, &id );
+				Connector *connector;
 					
-				startNode->addOutputConnector(connector);
-				endNode->addInputConnector(connector);
+				// HACK // FIXME // TODO
+				// for some strange reason the lists in the ItemDocument class the ID lists for items 
+				// get out of sync, so some id's are considered to be registered, but in fact they 
+				// have no assiciated items; this causes stange bugs when insterting subcircuits in the circuit.
+				// this is just a temporary fix; someone should get to the real cause of this problem and fix
+				// ItemDocument
+                                if ( icnd->connectorWithID( id ) ) {
+                                    kdWarning() << "Unregistering connector with ID: " << id << ". This should not delete any of your connections!" << endl;
+                                }
+				icnd->unregisterUID(id);
+				
+				// FIXME ICNDocument->type() used
+				// FIXME tons of dynamic_cast
+				if( icnd->type() == Document::dt_circuit ) {
+					connector = new ElectronicConnector( 
+								dynamic_cast<ECNode *>(startNode), 
+								dynamic_cast<ECNode *>(endNode), icnd, &id );
+					(dynamic_cast<ECNode *>(startNode))->addConnector(connector);
+					(dynamic_cast<ECNode *>(endNode))->addConnector(connector);
+				} else {
+					connector = new FlowConnector( 
+								dynamic_cast<FPNode *>(startNode),
+								dynamic_cast<FPNode *>(endNode), icnd, &id );
+					(dynamic_cast<FPNode *>(startNode))->addOutputConnector(connector);
+					(dynamic_cast<FPNode *>(endNode))->addInputConnector(connector);
+				}
 			}
 		}
 		for ( ConnectorDataMap::iterator it = m_connectorDataMap.begin(); it != connectorEnd; ++it )
@@ -1141,18 +1167,27 @@ void ItemDocumentData::addNodes( const NodeList &nodeList )
 
 void ItemDocumentData::addItemData( ItemData itemData, QString id )
 {
+        if ( m_itemDataMap.contains( id ) ) {
+            kdWarning() << "Overwriting item: " << id << endl;
+        }
 	m_itemDataMap[id] = itemData;
 }
 
 
 void ItemDocumentData::addConnectorData( ConnectorData connectorData, QString id )
 {
+        if ( m_connectorDataMap.contains( id ) ) {
+            kdWarning() << "Overwriting connector: " << id << endl;
+        }
 	m_connectorDataMap[id] = connectorData;
 }
 
 
 void ItemDocumentData::addNodeData( NodeData nodeData, QString id )
 {
+        if ( m_nodeDataMap.contains( id ) ) {
+            kdWarning() << "Overwriting node: " << id << endl;
+        }
 	m_nodeDataMap[id] = nodeData;
 }
 //END class ItemDocumentData
