@@ -200,6 +200,7 @@ Sidebar::Sidebar (KMultiTabBar::KMultiTabBarPosition pos, MainWindow *mainwin, Q
   , m_lastSize (0)
 {
   setSidebarPosition( pos );
+  setFocusPolicy( NoFocus );
   hide ();
 }
 
@@ -274,6 +275,30 @@ ToolView *Sidebar::addWidget (const QPixmap &icon, const QString &text, ToolView
   return widget;
 }
 
+void Sidebar::updateMinimumSize()
+{
+// 	kdDebug() << "layout()->margin()="<<layout()->margin()<<endl;
+	
+	QSize minSize;
+	
+	QValueList<ToolView*>::iterator end = m_toolviews.end();
+	for ( QValueList<ToolView*>::iterator it = m_toolviews.begin(); it != end; ++it )
+	{
+		QSize s = (*it)->childrenRect().size();
+		minSize = minSize.expandedTo( s );
+// 		kdDebug() << "s="<<s<<"(*it)->minimumSize()="<<(*it)->minimumSize()<<endl;
+// 		kdDebug() << "(*it)->layout()->margin()="<<(*it)->margin()<<endl;
+	}
+	
+	minSize.setWidth( minSize.width() - 30 );
+	minSize.setHeight( minSize.height() - 30 );
+	
+	for ( QValueList<ToolView*>::iterator it = m_toolviews.begin(); it != end; ++it )
+	{
+		(*it)->setMinimumSize( minSize );
+	}
+}
+
 bool Sidebar::removeWidget (ToolView *widget)
 {
   if (!m_widgetToId.contains(widget))
@@ -855,9 +880,16 @@ void MainWindow::saveSession (KConfig *config, const QString &group)
     m_sidebars[i]->saveSession (config);
 }
 
+void KateMDI::MainWindow::updateSidebarMinimumSizes( )
+{
+// 	for (unsigned int i=0; i < 4; ++i)
+// 		m_sidebars[i]->updateMinimumSize();
+	m_sidebars[KMultiTabBar::Right]->updateMinimumSize();
+}
+
 //END MAIN WINDOW
 
-} // namespace KateMDI
+}
 
 #include "katemdi.moc"
 
