@@ -32,7 +32,7 @@
 #include <qpopupmenu.h>
 #include <qwhatsthis.h>
 
-#include <assert.h>
+#include <cassert>
 
 //BEGIN class LinkerOptions
 LinkerOptions::LinkerOptions()
@@ -179,13 +179,12 @@ void ProcessingOptions::domElementToProcessingOptions( const QDomElement & eleme
 
 
 //BEGIN class ProjectItem
-ProjectItem::ProjectItem( ProjectItem * parent, Type type, ProjectManager * projectManager, KTechlab * ktechlab )
+ProjectItem::ProjectItem( ProjectItem * parent, Type type, ProjectManager * projectManager )
 	: QObject()
 {
 	m_pParent = parent;
 	m_pILVItem = 0l;
 	m_pProjectManager = projectManager;
-	p_ktechlab = ktechlab;
 	m_type = type;
 }
 
@@ -199,7 +198,6 @@ ProjectItem::~ProjectItem()
 	m_children.clear();
 	
 	delete m_pILVItem;
-	m_pILVItem = 0l;
 }
 
 
@@ -532,7 +530,7 @@ void ProjectItem::upload( ProcessOptionsList * pol )
 {
 	build( pol );
 	
-	ProgrammerDlg * dlg = new ProgrammerDlg( microID(), (QWidget*)p_ktechlab, "Programmer Dlg" );
+	ProgrammerDlg * dlg = new ProgrammerDlg( microID(), (QWidget*)KTechlab::self(), "Programmer Dlg" );
 	
 	dlg->exec();
 	if ( !dlg->isAccepted() )
@@ -636,7 +634,7 @@ bool ProjectItem::closeOpenFiles()
 
 void ProjectItem::addFiles()
 {
-	KURL::List urls = p_ktechlab->getFileURLs();
+	KURL::List urls = KTechlab::self()->getFileURLs();
 	const KURL::List::iterator end = urls.end();
 	for ( KURL::List::iterator it = urls.begin(); it != end; ++ it)
 		addFile(*it);
@@ -676,7 +674,7 @@ void ProjectItem::addFile( const KURL & url )
 			return;
 	}
 	
-	ProjectItem * item = new ProjectItem( this, FileType, m_pProjectManager, p_ktechlab );
+	ProjectItem * item = new ProjectItem( this, FileType, m_pProjectManager );
 	item->setURL(url);
 	addChild(item);
 }
@@ -726,7 +724,7 @@ void ProjectItem::domElementToItem( const QDomElement & element, const KURL & ba
 	QString name = element.attribute( "name", QString::null );
 	KURL url( baseURL, element.attribute( "url", QString::null ) );
 	
-	ProjectItem * createdItem = new ProjectItem( this, type, m_pProjectManager, p_ktechlab );
+	ProjectItem * createdItem = new ProjectItem( this, type, m_pProjectManager );
 	createdItem->setName( name );
 	createdItem->setURL( url );
 	
@@ -761,8 +759,8 @@ void ProjectItem::domElementToItem( const QDomElement & element, const KURL & ba
 
 
 //BEGIN class ProjectInfo
-ProjectInfo::ProjectInfo( ProjectManager * projectManager, KTechlab * ktechlab )
-	: ProjectItem( 0l, ProjectItem::ProjectType, projectManager, ktechlab )
+ProjectInfo::ProjectInfo( ProjectManager * projectManager )
+	: ProjectItem( 0l, ProjectItem::ProjectType, projectManager )
 {
 	m_microID = QString::null;
 }
@@ -854,9 +852,6 @@ bool ProjectInfo::save()
 	QDomElement root = doc.createElement("project");
 	doc.appendChild(root);
 	
-// 	root.appendChild( LinkerOptions::toDomElement(doc) );
-// 	root.appendChild( ProcessingOptions::toDomElement(doc) );
-	
 	m_children.remove( (ProjectItem*)0l );
 	ProjectItemList::const_iterator end = m_children.end();
 	for ( ProjectItemList::const_iterator it = m_children.begin(); it != end; ++it )
@@ -866,7 +861,7 @@ bool ProjectInfo::save()
 	stream << doc.toString();
 	file.close();
 	
-	(static_cast<RecentFilesAction*>(p_ktechlab->action("project_open_recent")))->addURL(m_url);
+	(static_cast<RecentFilesAction*>(KTechlab::self()->action("project_open_recent")))->addURL(m_url);
 	
 	return true;
 }
@@ -889,22 +884,20 @@ bool ProjectInfo::saveAndClose()
 //BEGIN class ProjectManager
 ProjectManager * ProjectManager::m_pSelf = 0l;
 
-ProjectManager * ProjectManager::self( KTechlab * ktl, KateMDI::ToolView * parent )
+ProjectManager * ProjectManager::self( KateMDI::ToolView * parent )
 {
 	if ( !m_pSelf )
 	{
-		assert(ktl);
 		assert(parent);
-		m_pSelf = new ProjectManager( ktl, parent );
+		m_pSelf = new ProjectManager( parent );
 	}
 	return m_pSelf;
 }
 
 
-ProjectManager::ProjectManager( KTechlab * ktl, KateMDI::ToolView * parent )
+ProjectManager::ProjectManager( KateMDI::ToolView * parent )
 	: ItemSelector( parent, "Project Manager" ),
-	m_pCurrentProject(0l),
-	p_ktechlab(ktl)
+	m_pCurrentProject(0l)
 {
 	QWhatsThis::add( this, i18n("Displays the list of files in the project.\nTo open or close a project, use the \"Project\" menu. Right click on a file to remove it from the project") );
 	
@@ -930,7 +923,7 @@ void ProjectManager::slotNewProject()
 
 	if ( newProjectDlg->accepted() )
 	{
-		m_pCurrentProject = new ProjectInfo( this, p_ktechlab );
+		m_pCurrentProject = new ProjectInfo( this );
 		m_pCurrentProject->setName( newProjectDlg->projectName() );
 		m_pCurrentProject->setURL( newProjectDlg->location() + m_pCurrentProject->name().lower() + ".ktechlab" );
 		
@@ -955,8 +948,10 @@ void ProjectManager::slotProjectOptions()
 
 void ProjectManager::slotOpenProject()
 {
-    KURL url = KFileDialog::getOpenURL(QString::null,
-	        "*.ktechlab|KTechlab Project(*.ktechlab)\n*|All Files", this, i18n("Open Location"));
+	QString filter;
+	filter = QString("*.ktechlab|%1 (*.ktechlab)\n*|%2").arg( i18n("KTechlab Project") ).arg( i18n("All Files") );
+	
+    KURL url = KFileDialog::getOpenURL(QString::null, filter, this, i18n("Open Location"));
 	
     if ( url.isEmpty() )
 		return;
@@ -973,7 +968,7 @@ void ProjectManager::slotOpenProject( const KURL & url )
 	if ( !slotCloseProject() )
 		return;
 	
-	m_pCurrentProject = new ProjectInfo( this, p_ktechlab );
+	m_pCurrentProject = new ProjectInfo( this );
 	
 	if ( !m_pCurrentProject->open(url) )
 	{
@@ -982,11 +977,11 @@ void ProjectManager::slotOpenProject( const KURL & url )
 		return;
 	}
 	
-	RecentFilesAction * rfa = static_cast<RecentFilesAction*>(p_ktechlab->action("project_open_recent"));
+	RecentFilesAction * rfa = static_cast<RecentFilesAction*>(KTechlab::self()->action("project_open_recent"));
 	rfa->addURL( m_pCurrentProject->url() );
 	
 	if ( KTLConfig::raiseItemSelectors() )
-		p_ktechlab->showToolView( p_ktechlab->toolView( toolViewIdentifier() ) );
+		KTechlab::self()->showToolView( KTechlab::self()->toolView( toolViewIdentifier() ) );
 	
 	updateActions();
 	emit projectOpened();
@@ -1031,7 +1026,7 @@ void ProjectManager::slotCreateSubproject()
 				break;
 		}
 		
-		ProjectItem * subproject = new ProjectItem( currentProject(), type, this, p_ktechlab );
+		ProjectItem * subproject = new ProjectItem( currentProject(), type, this );
 		subproject->setURL( dlg->targetFile() );
 		
 		currentProject()->addChild(subproject);
@@ -1048,14 +1043,14 @@ void ProjectManager::updateActions()
 {
 	bool projectIsOpen = m_pCurrentProject;
 	
-	p_ktechlab->action("project_create_subproject")->setEnabled( projectIsOpen );
-	p_ktechlab->action("project_export_makefile")->setEnabled( projectIsOpen );
-	p_ktechlab->action("subproject_add_existing_file")->setEnabled( projectIsOpen );
-	p_ktechlab->action("subproject_add_current_file")->setEnabled( projectIsOpen );
-// 	p_ktechlab->action("project_options")->setEnabled( projectIsOpen );
-	p_ktechlab->action("project_close")->setEnabled( projectIsOpen );
-	p_ktechlab->action("project_add_existing_file")->setEnabled( projectIsOpen );
-	p_ktechlab->action("project_add_current_file")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("project_create_subproject")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("project_export_makefile")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("subproject_add_existing_file")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("subproject_add_current_file")->setEnabled( projectIsOpen );
+// 	KTechlab::self()->action("project_options")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("project_close")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("project_add_existing_file")->setEnabled( projectIsOpen );
+	KTechlab::self()->action("project_add_current_file")->setEnabled( projectIsOpen );
 }
 
 
@@ -1193,7 +1188,7 @@ void ProjectManager::slotContextMenuRequested( QListViewItem * item, const QPoin
 {
 	QString popupName;
 	ILVItem * ilvItem = dynamic_cast<ILVItem*>(item);
-	KAction * linkerOptionsAct = p_ktechlab->action("project_item_linker_options");
+	KAction * linkerOptionsAct = KTechlab::self()->action("project_item_linker_options");
 	linkerOptionsAct->setEnabled(false);
 	
 	if ( !m_pCurrentProject )
@@ -1244,10 +1239,10 @@ void ProjectManager::slotContextMenuRequested( QListViewItem * item, const QPoin
 	}
 	
 	bool haveFocusedDocument = DocManager::self()->getFocusedDocument();
-	p_ktechlab->action("subproject_add_current_file")->setEnabled( haveFocusedDocument );
-	p_ktechlab->action("project_add_current_file")->setEnabled( haveFocusedDocument );
+	KTechlab::self()->action("subproject_add_current_file")->setEnabled( haveFocusedDocument );
+	KTechlab::self()->action("project_add_current_file")->setEnabled( haveFocusedDocument );
 	
-	QPopupMenu *pop = static_cast<QPopupMenu*>(p_ktechlab->factory()->container( popupName, p_ktechlab ));
+	QPopupMenu *pop = static_cast<QPopupMenu*>(KTechlab::self()->factory()->container( popupName, KTechlab::self() ));
 	if (pop)
 		pop->popup(pos);
 }
