@@ -34,9 +34,9 @@
 #include <ktempfile.h>
 
 
-TextDocument *TextDocument::constructTextDocument( const QString& caption, KTechlab *parent, const char *name )
+TextDocument *TextDocument::constructTextDocument( const QString& caption, const char *name )
 {
-	TextDocument *textDocument = new TextDocument( caption, parent, name);
+	TextDocument *textDocument = new TextDocument( caption, name);
 	if( textDocument->m_constructorSuccessful )
 		return textDocument;
 	delete textDocument;
@@ -44,8 +44,8 @@ TextDocument *TextDocument::constructTextDocument( const QString& caption, KTech
 }
 
 
-TextDocument::TextDocument( const QString &caption, KTechlab *ktechlab, const char *name )
-	: Document( caption, ktechlab, name ),
+TextDocument::TextDocument( const QString &caption, const char *name )
+	: Document( caption, name ),
 	  m_doc(0)
 {
 	m_constructorSuccessful = false;
@@ -66,7 +66,7 @@ TextDocument::TextDocument( const QString &caption, KTechlab *ktechlab, const ch
 	KLibFactory *factory = KLibLoader::self()->factory("libkatepart");
 	if(!factory)
 	{
-		KMessageBox::sorry( ktechlab, i18n("Libkatepart not available for constructing editor") );
+		KMessageBox::sorry( KTechlab::self(), i18n("Libkatepart not available for constructing editor") );
 		return;
 	}
 	m_doc = (Kate::Document*)(KTextEditor::Document *)factory->create( 0L, "kate", "KTextEditor::Document");
@@ -93,26 +93,25 @@ TextDocument::TextDocument( const QString &caption, KTechlab *ktechlab, const ch
 
 TextDocument::~TextDocument()
 {
-	if( !m_constructorSuccessful )
-		return; 
+	if( !m_constructorSuccessful ) return;
 
 	debugStop();
 	
-	ViewList::iterator end = m_viewList.end();
-	for ( ViewList::iterator it = m_viewList.begin(); it != end; ++it )
+	if ( KTechlab::self() )
 	{
-		if ( TextView * tv = dynamic_cast<TextView*>( (View*)*it) )
+		ViewList::iterator end = m_viewList.end();
+		for ( ViewList::iterator it = m_viewList.begin(); it != end; ++it )
 		{
-			Kate::View * kv = tv->kateView();
-			p_ktechlab->factory()->removeClient( kv );
+			if ( TextView * tv = dynamic_cast<TextView*>( (View*)*it) )
+			{
+				Kate::View * kv = tv->kateView();
+				KTechlab::self()->factory()->removeClient( kv );
+			}
 		}
 	}
 	
 	delete m_doc;
-	m_doc = 0l;
-	
 	delete m_pDocumentIface;
-	m_pDocumentIface = 0l;
 }
 
 
@@ -435,13 +434,13 @@ void TextDocument::convertToAssembly()
 		return;
 	}
 	
-	OutputMethodDlg dlg( i18n("Assembly Code Output"), url(), showPICSelect, p_ktechlab );
+	OutputMethodDlg dlg( i18n("Assembly Code Output"), url(), showPICSelect, KTechlab::self() );
 	
 	if ( m_guessedCodeType == TextDocument::ct_c )
 		dlg.microSelect()->setAllowedAsmSet( AsmInfo::PIC14 | AsmInfo::PIC16 );
 	
 	dlg.setOutputExtension(".asm");
-	dlg.setFilter("*.asm *.src *.inc|Assembly Code (*.asm, *.src, *.inc)\n*|All Files");
+	dlg.setFilter( QString("*.asm *.src *.inc|%1 (*.asm, *.src, *.inc)\n*|%2").arg(i18n("Assembly Code")).arg(i18n("All Files")) );
 	dlg.exec();
 	if (!dlg.isAccepted())
 		return;
@@ -463,8 +462,10 @@ void TextDocument::convertToHex()
 		filePath = outputFilePath(".microbe");
 	
 	else if ( m_guessedCodeType == TextDocument::ct_asm )
+	{
 		filePath = outputFilePath(".asm");
-	
+		showPICSelect = true;	// FIXME if we use shared libs, then we need the pic type
+	}
 	else if ( m_guessedCodeType == TextDocument::ct_c )
 	{
 		filePath = outputFilePath(".c");
@@ -477,9 +478,9 @@ void TextDocument::convertToHex()
 		return;
 	}
 	
-	OutputMethodDlg dlg( i18n("Hex Code Output"), url(), showPICSelect, p_ktechlab );
+	OutputMethodDlg dlg( i18n("Hex Code Output"), url(), showPICSelect, KTechlab::self() );
 	dlg.setOutputExtension(".hex");
-	dlg.setFilter("*.hex|Hex (*.hex)\n*|All Files");
+	dlg.setFilter( QString("*.hex|Hex (*.hex)\n*|%1").arg(i18n("All Files")) );
 	
 	if ( m_guessedCodeType == TextDocument::ct_c )
 		dlg.microSelect()->setAllowedAsmSet( AsmInfo::PIC14 | AsmInfo::PIC16 );
@@ -530,7 +531,7 @@ void TextDocument::convertToPIC()
 			return;
 	}
 	
-	ProgrammerDlg * dlg = new ProgrammerDlg( picID, (QWidget*)p_ktechlab, "Programmer Dlg" );
+	ProgrammerDlg * dlg = new ProgrammerDlg( picID, (QWidget*)KTechlab::self(), "Programmer Dlg" );
 	
 	if ( m_guessedCodeType == TextDocument::ct_c )
 		dlg->microSelect()->setAllowedAsmSet( AsmInfo::PIC14 | AsmInfo::PIC16 );	
@@ -560,8 +561,8 @@ void TextDocument::print()
 
 void TextDocument::slotSelectionmChanged()
 {
-	p_ktechlab->action( "edit_cut" )->setEnabled( m_doc->hasSelection () );
-	p_ktechlab->action( "edit_copy" )->setEnabled( m_doc->hasSelection () );
+	KTechlab::self()->action( "edit_cut" )->setEnabled( m_doc->hasSelection () );
+	KTechlab::self()->action( "edit_copy" )->setEnabled( m_doc->hasSelection () );
 }
 
 
@@ -585,6 +586,9 @@ IntList TextDocument::bookmarkList() const
 
 void TextDocument::slotUpdateMarksInfo()
 {
+	if ( !KTechlab::self() )
+		return;
+	
 	if ( activeView() )
 		textView()->slotUpdateMarksInfo();
 	
@@ -593,7 +597,7 @@ void TextDocument::slotUpdateMarksInfo()
 #endif
 	
 	// Update our list of bookmarks in the menu
-	p_ktechlab->unplugActionList("bookmark_actionlist");
+	KTechlab::self()->unplugActionList("bookmark_actionlist");
 	m_bookmarkActions.clear();
 	
 	QPtrList<KTextEditor::Mark> markList = m_doc->marks();
@@ -610,7 +614,7 @@ void TextDocument::slotUpdateMarksInfo()
 		}
 	}
 	
-	p_ktechlab->plugActionList( "bookmark_actionlist", m_bookmarkActions );
+	KTechlab::self()->plugActionList( "bookmark_actionlist", m_bookmarkActions );
 }
 
 
